@@ -1,8 +1,27 @@
-/* $XConsortium: Dialog.c,v 1.45 91/03/21 14:38:27 dave Exp $ */
+/* $TOG: Dialog.c /main/49 1998/02/06 12:45:36 kaleb $ */
 
-/***********************************************************
-Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
-and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
+/************************************************************************
+
+Copyright 1987, 1988, 1994, 1998  The Open Group
+
+All Rights Reserved.
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of The Open Group shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from The Open Group.
+
+
+Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
@@ -10,7 +29,7 @@ Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, 
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in 
-supporting documentation, and that the names of Digital or MIT not be
+supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
 software without specific, written prior permission.  
 
@@ -22,7 +41,9 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
-******************************************************************/
+This file contains modifications for XawPlus, Roland Krause 2000
+
+************************************************************************/
 
 /* NOTE: THIS IS NOT A WIDGET!  Rather, this is an interface to a widget.
    It implements policy, and gives a (hopefully) easier-to-use interface
@@ -163,33 +184,33 @@ static void ClassInitialize()
  *
  ***********************************************************************/
 
-static void Initialize(request, new, args, n_args)
+static void Initialize(request, new, args, num_args)
 Widget request, new;
 ArgList args;
-Cardinal *n_args;
+Cardinal *num_args;
 {
     DialogWidget dw = (DialogWidget)new;
     Arg arglist[9];
-    Cardinal num_args = 0;
+    Cardinal arg_cnt = 0;
 
-    XtSetArg(arglist[num_args], XtNborderWidth, 0); num_args++;
-    XtSetArg(arglist[num_args], XtNleft, XtChainLeft); num_args++;
+    XtSetArg(arglist[arg_cnt], XtNborderWidth, 0); arg_cnt++;
+    XtSetArg(arglist[arg_cnt], XtNleft, XtChainLeft); arg_cnt++;
 
     if (dw->dialog.icon != None) {
-	XtSetArg(arglist[num_args], XtNbitmap, dw->dialog.icon); num_args++;
-	XtSetArg(arglist[num_args], XtNright, XtChainLeft); num_args++;
-	XtSetArg(arglist[num_args], XtNclipMask, dw->dialog.clipMask); num_args++;
+	XtSetArg(arglist[arg_cnt], XtNbitmap, dw->dialog.icon); arg_cnt++;
+	XtSetArg(arglist[arg_cnt], XtNright, XtChainLeft); arg_cnt++;
+	XtSetArg(arglist[arg_cnt], XtNclipMask, dw->dialog.clipMask); arg_cnt++;
 	dw->dialog.iconW = XtCreateManagedWidget("icon", labelWidgetClass,
-						 new, arglist, num_args );
-	num_args = 2;
-	XtSetArg(arglist[num_args], XtNfromHoriz, dw->dialog.iconW);num_args++;
+						 new, arglist, arg_cnt );
+	arg_cnt = 2;
+	XtSetArg(arglist[arg_cnt], XtNfromHoriz, dw->dialog.iconW); arg_cnt++;
     } else dw->dialog.iconW = (Widget)NULL;
 
-    XtSetArg(arglist[num_args], XtNlabel, dw->dialog.label); num_args++;
-    XtSetArg(arglist[num_args], XtNright, XtChainRight); num_args++;
+    XtSetArg(arglist[arg_cnt], XtNlabel, dw->dialog.label); arg_cnt++;
+    XtSetArg(arglist[arg_cnt], XtNright, XtChainRight); arg_cnt++;
 
     dw->dialog.labelW = XtCreateManagedWidget( "label", labelWidgetClass,
-					      new, arglist, num_args);
+					      new, arglist, arg_cnt);
 
     if (dw->dialog.iconW != (Widget)NULL &&
 	(dw->dialog.labelW->core.height < dw->dialog.iconW->core.height)) {
@@ -208,8 +229,10 @@ Cardinal *n_args;
  *
  ***********************************************************************/
 
-static void ConstraintInitialize(request, new)
+static void ConstraintInitialize(request, new, args, num_args)
 Widget request, new;
+ArgList args;
+Cardinal *num_args;
 {
     DialogWidget dw = (DialogWidget)new->core.parent;
     DialogConstraints constraint = (DialogConstraints)new->core.constraints;
@@ -358,7 +381,7 @@ Cardinal * num_args;
   Arg a[1];
   String s;
   DialogWidget src = (DialogWidget) w;
-  register int i;
+  int i;
   
   for (i=0; i < *num_args; i++)
     if (streq(args[i].name, XtNvalue)) {
@@ -397,8 +420,8 @@ Widget w;
     XtSetArg(arglist[num_args], XtNleft, XtChainLeft);            num_args++;
     XtSetArg(arglist[num_args], XtNright, XtChainRight);          num_args++;
 
-    dw->dialog.valueW = XtCreateWidget("value",asciiTextWidgetClass,
-				       w, arglist, num_args);
+    dw->dialog.valueW = XtCreateWidget("value", asciiTextWidgetClass,
+				     w, arglist, num_args);
 
     /* if the value widget is being added after buttons,
      * then the buttons need new layout constraints.
@@ -431,7 +454,7 @@ Widget w;
 
 void XawDialogAddButton(dialog, name, function, param)
 Widget dialog;
-_Xconst char *name;
+String name;
 XtCallbackProc function;
 XtPointer param;
 {
@@ -441,7 +464,7 @@ XtPointer param;
     Widget button;
 
     button = XtCreateManagedWidget( name, commandWidgetClass, dialog, 
-				    NULL, (Cardinal) 0 );
+				    (ArgList)NULL, (Cardinal)0 );
 
     if (function != NULL)	/* don't add NULL callback func. */
         XtAddCallback(button, XtNcallback, function, param);
