@@ -1,8 +1,27 @@
-/* $XConsortium: Label.c,v 1.92 91/07/24 16:56:28 converse Exp $ */
+/* $TOG: Label.c /main/98 1998/02/06 12:46:24 kaleb $ */
 
-/***********************************************************
-Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
-and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
+/*************************************************************************
+
+Copyright 1987, 1988, 1994, 1998  The Open Group
+
+All Rights Reserved.
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of The Open Group shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from The Open Group.
+
+
+Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
@@ -10,7 +29,7 @@ Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, 
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in 
-supporting documentation, and that the names of Digital or MIT not be
+supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
 software without specific, written prior permission.  
 
@@ -22,17 +41,19 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
-3D extensions and XPM support added : Roland Krause 1996, 1997
-Changes for the new XPM conversion mechanism : Roland Krause 1999
-New mechanism to truncate long label strings : Roland Krause 1999
+This file contains EXTENSIVE modifications for XawPlus, Roland Krause 2000
 
-******************************************************************/
+****************************************************************************/
 
 /*
  * Label.c - Label widget
  *
  */
 
+#include <stdio.h>
+#include <ctype.h>
+#include <stdlib.h>
+
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Xos.h>
@@ -41,15 +62,14 @@ New mechanism to truncate long label strings : Roland Krause 1999
 #include <X11/XawPlus/StrToPmap.h>
 #include <X11/Xmu/Converters.h>
 #include <X11/Xmu/Drawing.h>
-#include <stdio.h>
-#include <ctype.h>
+
+#ifdef X_NOT_STDC_ENV
+int abs();
+#endif
 
 #define streq(a,b) (strcmp( (a), (b) ) == 0)
 #define MULTI_LINE_LABEL 32767
 
-#ifdef CRAY
-#define WORD64
-#endif
 
 /****************************************************************
  *
@@ -65,6 +85,8 @@ static XtResource resources[] = {
 	offset(label.foreground), XtRString, XtDefaultForeground},
     {XtNfont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
 	offset(label.font),XtRString, XtDefaultFont},
+    {XtNfontSet,  XtCFontSet, XtRFontSet, sizeof(XFontSet ),
+        offset(label.fontset),XtRString, XtDefaultFontSet},
     {XtNlabel,  XtCLabel, XtRString, sizeof(String),
 	offset(label.label), XtRString, NULL},
     {XtNencoding, XtCEncoding, XtRUnsignedChar, sizeof(unsigned char),
@@ -146,141 +168,379 @@ LabelClassRec labelClassRec = {
     /* display_accelerator	*/	XtInheritDisplayAccelerator,
     /* extension		*/	NULL
   },
-/* add3dExt fields initialization */
-  {
-    /* unused			*/	0
-  },
 /* Simple class fields initialization */
   {
     /* change_sensitive		*/	XtInheritChangeSensitive
+  },
+/* Label class fields initialization */
+  {
+    /* ignore 			*/	0
   }
 };
+
 WidgetClass labelWidgetClass = (WidgetClass)&labelClassRec;
+
 /****************************************************************
  *
  * Private Procedures
  *
  ****************************************************************/
 
-#ifndef WORD64
-#define TXT16 XChar2b
-#else
+/* String functions, that works with 16 bit characters.
+ * These functions are used to handle strings with 16 bit encoding.
+ */
 
-#define TXT16 char
+typedef unsigned short char16;	/* The data type used for 16 bit character strings */
 
-static XChar2b *buf2b;
-static int buf2blen = 0;
-
-_XawLabelWidth16(fs, str, n)
-    XFontStruct *fs;
-    char *str;
-    int	n;
+/* Determine the length of the string in char16's
+ */
+static int str16len(str16)
+char16 *str16;
 {
-    int i;
-    XChar2b *ptr;
+   int len = 0;
 
-    if (n > buf2blen) {
-	buf2b = (XChar2b *)XtRealloc((char *)buf2b, n * sizeof(XChar2b));
-	buf2blen = n;
-    }
-    for (ptr = buf2b, i = n; --i >= 0; ptr++) {
-	ptr->byte1 = *str++;
-	ptr->byte2 = *str++;
-    }
-    return XTextWidth16(fs, buf2b, n);
+   while (*str16++) len++;
+   return len;
 }
 
-_XawLabelDraw16(dpy, d, gc, x, y, str, n)
-    Display *dpy;
-    Drawable d;
-    GC gc;
-    int x, y;
-    char *str;
-    int n;
+/* Search the character ch in str16 and return a
+ * pointer to this character. Otherwise return NULL.
+ */
+static char16 *str16chr(str16, ch)
+char16 *str16;
+char ch;
 {
-    int i;
-    XChar2b *ptr;
+   char *p;
 
-    if (n > buf2blen) {
-	buf2b = (XChar2b *)XtRealloc((char *)buf2b, n * sizeof(XChar2b));
-	buf2blen = n;
-    }
-    for (ptr = buf2b, i = n; --i >= 0; ptr++) {
-	ptr->byte1 = *str++;
-	ptr->byte2 = *str++;
-    }
-    XDrawString16(dpy, d, gc, x, y, buf2b, n);
+   while (*str16)
+   {
+     p = (char *)str16;
+     if ((*p++ == '\0') && (*p == ch)) return(str16);
+     str16++;
+   }
+   return NULL;
 }
 
-#define XTextWidth16 _XawLabelWidth16
-#define XDrawString16 _XawLabelDraw16
+/* Copy the string from src to dest.
+ */
+static void str16cpy(dest, src)
+char16 *dest, *src;
+{
+   while (*src) *dest++ = *src++;
+   *dest = 0;		/* To terminate the string */
+}
 
-#endif /* WORD64 */
+/* Copy n characters from src to dest or
+ * until the end of src.
+ */
+static void str16ncpy(dest, src, n)
+char16 *dest, *src;
+size_t n;
+{
+   while ((n > 0) && *src)
+   {
+     *dest++ = *src++;
+     n--;
+   }
+   *dest = 0;	/* We always terminate the string here */
+}
+
+/* Copy the content of src behind dest.
+ * src and dest has to be null terminated !
+ */
+static void str16cat(dest, src)
+char16 *dest, *src;
+{
+   while (*dest) dest++;	  /* search the end of the string */
+   while (*src) *dest++ = *src++; /* copy the other behind */
+   *dest = 0;			  /* and terminate the string */
+}
+
+/* Determine the numer of bytes of the
+ * multibyte character pointed to by str.
+ * We don't check if the multibyte code is
+ * valid !
+ */
+static int mbCharLen(str)
+char *str;
+{
+   unsigned char c = (unsigned char)*str;
+   unsigned char mask = 0x80;
+   int count = 0;
+
+   if (c)
+   {
+     while (c & mask)
+     {
+        count++;
+        mask >>= 1;
+     }
+
+     switch (count)
+     {
+	case 0:	 return 1;	/* Character is one byte long */
+	case 1:  return 0;	/* No multibyte string or wrong position */
+	default: return count;
+     }
+   }
+   return 0;	/* End of string reached */
+}
+
+/* Determine the string length in characters (not in byte!)
+ * We don't count the trailing null byte here.
+ */
+static int mbStrLen(str)
+char *str;
+{
+   char *p = str;
+   int clen, len = 0;
+
+   while ((clen = mbCharLen(p)) > 0)
+   {
+      len++;
+      p += clen;
+   }
+   return len;
+}
+
+/* Conversion of an UTF8 coded string into UCS2/UNICODE.
+ * If the encoding of the character is not representable
+ * in two bytes, the tilde sign ~ is written into the
+ * result string at this position.
+ * For an illegal UTF8 code an asterix * is stored in
+ * the result string.
+ *
+ * This function allocates memory for the result, which
+ * has to be released by the caller.
+ */
+
+static char16 *UTF8toUCS2(str)
+char *str;
+{
+   char16 *str16, *p16, testINTEL = 1, c16;
+   int    clen, cInString;
+   char   *p;
+
+  /* In the first step we try to determine the string
+   * length in characters.
+   */
+   cInString = mbStrLen(str);
+   cInString++;		/* For the terminating null */
+
+   /* Now we need memory for our conversion result */
+
+   str16 = (char16 *)XtMalloc(cInString * sizeof(char16));
+   if (str16)
+   {
+     /* Start the conversion: Determine the number of bytes
+      * for the next character, decode it and store the
+      * result in our result string
+      */
+      p   = str;
+      p16 = str16;
+      while ((clen = mbCharLen(p)) > 0)
+      {
+	 switch (clen)
+	 {
+	    case 1: *p16 = (char16)*p;
+		    break;
+
+	    case 2: c16 = (p[1] & 0x3f) + ((p[0] & 0x1f) << 6);
+		    *p16 = ((c16 > 0x7f) && ((p[1] & 0xC0) == 0x80)) ? c16 : '*';
+		    break;
+
+	    case 3: c16 = (p[2] & 0x3f) + ((p[1] & 0x3f) << 6) + ((p[0] & 0xf) << 12);
+		    *p16 = ((c16 > 0x7ff) && ((p[1] & 0xC0) == 0x80) && ((p[2] & 0xC0) == 0x80)) ? c16 : '*';
+		    break;
+
+	    default: *p16 = '~';	/* character code is greater than 0xffff */
+	 }
+	 p16++;				/* Jump to the next character */
+	 p += clen;
+      }
+      *p16 = 0;				/* String termination */
+
+      /* Swap the bytes, if we are on a machine with an INTEL architecture */
+
+      if (*((char *)&testINTEL))
+      {
+	 char *src, *dest, c;
+
+	 src = dest = (char *)str16;
+	 src++;
+	 while (*src || *dest)
+	 {
+	    c = *dest; *dest = *src; *src = c;
+	    src += 2; dest += 2;
+	 }
+      }
+      return str16;
+   }
+   return NULL;
+}
+
+/* ----------------------------------------------------------------------------- */
+
+/* Calculate width and height of the label for 8 bit characters
+ */
+static void SetWidthAndHeight8Bit(lw)
+LabelWidget lw;
+{
+    XFontStruct *fs = lw->label.font;
+    int width;
+    char *nl, *label;
+
+    lw->label.label_height = fs->max_bounds.ascent + fs->max_bounds.descent;
+    if (lw->label.label == NULL)
+    {
+       lw->label.label_len = 0;
+       lw->label.label_width = 0;
+    }
+    else if ((nl = index(lw->label.label, '\n')) != NULL)
+    {
+       /* 8 bit multiline label */
+
+       lw->label.label_len = MULTI_LINE_LABEL;
+       lw->label.label_width = 0;
+       for (label = lw->label.label; nl != NULL; nl = index(label, '\n'))
+       {
+	  width = XTextWidth(fs, label, (int)(nl - label));
+	  if (width > (int)lw->label.label_width) lw->label.label_width = width;
+	  label = nl + 1;
+	  if (*label) lw->label.label_height +=
+			fs->max_bounds.ascent + fs->max_bounds.descent;
+       }
+       if (*label)
+       {
+	  width = XTextWidth(fs, label, strlen(label));
+	  if (width > (int) lw->label.label_width) lw->label.label_width = width;
+       }
+    }
+    else	/* 8 bit single line label */
+    {
+       lw->label.label_len = strlen(lw->label.label);
+       lw->label.label_width = XTextWidth(fs, lw->label.label, (int)lw->label.label_len);
+    }
+}
+
+/* Calculate width and height of the label for 16 bit characters
+ */
+static void SetWidthAndHeight2Byte(lw)
+LabelWidget lw;
+{
+    XFontStruct *fs = lw->label.font;
+    int width;
+    char16 *wnl, *label;
+
+    lw->label.label_height = fs->max_bounds.ascent + fs->max_bounds.descent;
+    if (lw->label.label == NULL)
+    {
+       lw->label.label_len = 0;
+       lw->label.label_width = 0;
+    }
+    else if ((wnl = str16chr((char16 *)lw->label.label, '\n')) != NULL)
+    {
+        /* 16 bit character multiline label */
+
+	lw->label.label_len = MULTI_LINE_LABEL;
+	lw->label.label_width = 0;
+	for (label = (char16 *)lw->label.label; wnl != NULL;
+	     wnl = str16chr(label, '\n'))
+	{
+	    width = XTextWidth16(fs, (XChar2b *)label, (int)(wnl - label));
+	    if (width > (int)lw->label.label_width) lw->label.label_width = width;
+	    label = wnl + 1;
+	    if (*label) lw->label.label_height +=
+			fs->max_bounds.ascent + fs->max_bounds.descent;
+    	}
+    	if (*label) 
+	{
+	   width = XTextWidth16(fs, (XChar2b *)label, (int)str16len(label));
+	   if (width > (int) lw->label.label_width) lw->label.label_width = width;
+	}
+    }
+    else	/* 16 bit character single line label */
+    {
+	lw->label.label_len = str16len((char16 *)lw->label.label);
+	lw->label.label_width = XTextWidth16(fs, (XChar2b *)lw->label.label,
+					      (int)lw->label.label_len);
+    }
+}
+
+/* Calculate width and height of the label for a multibyte string (UTF-8)
+ */
+static void SetWidthAndHeightMulti(lw)
+LabelWidget lw;
+{
+   XFontSet fset = lw->label.fontset;
+   XFontSetExtents *ext = XExtentsOfFontSet(fset);
+   char *label, *nl;
+   int width;
+
+   lw->label.label_height = ext->max_logical_extent.height;
+   if (lw->label.label == NULL)
+   {
+      lw->label.label_len = 0;
+      lw->label.label_width = 0;
+   }
+   else if ((nl = index(lw->label.label, '\n')) != NULL)
+   {
+      /* Multibyte multiline label */
+
+      lw->label.label_len = MULTI_LINE_LABEL;
+      lw->label.label_width = 0;
+      for (label = lw->label.label; nl != NULL; nl = index(label, '\n'))
+      {
+	  width = XmbTextEscapement(fset, label, (int)(nl - label));
+	  if (width > (int)lw->label.label_width) lw->label.label_width = width;
+	  label = nl + 1;
+	  if (*label) lw->label.label_height += ext->max_logical_extent.height;
+      }
+      if (*label)
+      {
+	 width = XmbTextEscapement(fset, label, strlen(label));
+         if (width > (int) lw->label.label_width) lw->label.label_width = width;
+      }
+   }
+   else	 /* Multibyte single line label */
+   {
+      lw->label.label_len = strlen(lw->label.label);
+      lw->label.label_width = XmbTextEscapement(fset, lw->label.label, (int)lw->label.label_len);
+   }
+}
 
 /*
  * Calculate width and height of displayed text in pixels
  */
 
 static void SetTextWidthAndHeight(lw)
-    LabelWidget lw;
+LabelWidget lw;
 {
-    register XFontStruct *fs = lw->label.font;
     Window root;
     int x, y;
     unsigned int width, height, bw, depth;
-    char *nl;
 
-    if (lw->label.pixmap != None) {		/* bitmap defined ? */
+    if (lw->label.pixmap != None)		/* bitmap defined ? */
+    {
 	if (XGetGeometry(XtDisplay(lw), lw->label.pixmap, &root, &x, &y,
-			 &width, &height, &bw, &depth)) {
+			 &width, &height, &bw, &depth))
+	{
 	    lw->label.label_height = height;
 	    lw->label.label_width  = width;
 	    lw->label.label_depth  = depth;
 	    return;
 	}
     }
-    lw->label.label_height = fs->max_bounds.ascent + fs->max_bounds.descent;
-    if (lw->label.label == NULL) {
-	lw->label.label_len = 0;
-	lw->label.label_width = 0;
-    }
-    else if ((nl = index(lw->label.label, '\n')) != NULL) {
-	char *label;
-	lw->label.label_len = MULTI_LINE_LABEL;
-	lw->label.label_width = 0;
-	for (label = lw->label.label; nl != NULL; nl = index(label, '\n')) {
-	    int width;
-
-	    if (lw->label.encoding)
-		width = XTextWidth16(fs, (TXT16*)label, (int)(nl - label)/2);
-	    else
-		width = XTextWidth(fs, label, (int)(nl - label));
-	    if (width > (int)lw->label.label_width)
-		lw->label.label_width = width;
-	    label = nl + 1;
-	    if (*label)
-		lw->label.label_height +=
-		    fs->max_bounds.ascent + fs->max_bounds.descent;
-	}
-	if (*label) {
-	    int width = XTextWidth(fs, label, strlen(label));
-
-	    if (lw->label.encoding)
-		width = XTextWidth16(fs, (TXT16*)label, (int)strlen(label)/2);
-	    else
-		width = XTextWidth(fs, label, strlen(label));
-	    if (width > (int) lw->label.label_width)
-		lw->label.label_width = width;
-	}
-    } else {
-	lw->label.label_len = strlen(lw->label.label);
-	if (lw->label.encoding)
-	    lw->label.label_width =
-		XTextWidth16(fs, (TXT16*)lw->label.label,
-			     (int) lw->label.label_len/2);
-	else
-	    lw->label.label_width =
-		XTextWidth(fs, lw->label.label, (int) lw->label.label_len);
+    /* Calculate the required width and height of the label depended
+     * on the character set. Possible character sets are
+     * ISO 8859, UNICODE or BMP (Basic Multilingual Plane defined in ISO 10646-1)
+     * or UTF8 (Multibyte code, also defined in ISO 10646-1)
+     */
+    if (lw->simple.international == True) SetWidthAndHeightMulti(lw);
+    else
+    {
+       if (lw->label.encoding) SetWidthAndHeight2Byte(lw);
+       else SetWidthAndHeight8Bit(lw);
     }
 }
 
@@ -296,9 +556,16 @@ static void GetnormalGC(lw)
     values.font			= lw->label.font->fid;
     values.graphics_exposures 	= False;
 
-    lw->label.normal_GC = XtGetGC((Widget)lw,
-		             GCForeground|GCBackground|GCFont|GCGraphicsExposures,
-		             &values);
+    /* Since Xmb/wcDrawString eats the font, I must use XtAllocateGC. */
+
+    if ( lw->simple.international == True )
+        lw->label.normal_GC = XtAllocateGC((Widget)lw, 0,
+			      GCForeground|GCBackground|GCGraphicsExposures,
+			      &values, GCFont, 0 );
+    else
+        lw->label.normal_GC = XtGetGC((Widget)lw,
+			      GCForeground|GCBackground|GCFont|GCGraphicsExposures,
+			      &values);
 }
 
 static void GetgrayGCs(lw)
@@ -306,19 +573,33 @@ static void GetgrayGCs(lw)
 {
     XGCValues	values;
 
-    values.foreground = lw->add3dExt.shadowColor;
+    values.foreground = lw->simple.shadowColor;
     values.background = lw->core.background_pixel;
     values.font	      = lw->label.font->fid;
     values.graphics_exposures = False;
-    lw->label.gray_GC = XtGetGC((Widget)lw,
-			  GCForeground|GCBackground|GCFont|GCGraphicsExposures,
-			  &values);
 
-    values.foreground  = lw->add3dExt.highlightColor;
-    lw->label.lgray_GC = XtGetGC((Widget)lw,
-			  GCForeground|GCBackground|GCFont|GCGraphicsExposures,
+    /* Since Xmb/wcDrawString eats the font, I must use XtAllocateGC. */
+
+    if ( lw->simple.international == True ) {
+      lw->label.gray_GC = XtAllocateGC((Widget)lw, 0,
+			  (unsigned)GCForeground|GCBackground|GCFont|GCGraphicsExposures,
+			  &values, GCFont, 0);
+
+      values.foreground  = lw->simple.highlightColor;
+      lw->label.lgray_GC = XtAllocateGC((Widget)lw, 0,
+			  (unsigned)GCForeground|GCBackground|GCFont|GCGraphicsExposures,
+			  &values, GCFont, 0);
+    }
+    else {
+      lw->label.gray_GC = XtGetGC((Widget)lw,
+			  (unsigned)GCForeground|GCBackground|GCFont|GCGraphicsExposures,
 			  &values);
 
+      values.foreground  = lw->simple.highlightColor;
+      lw->label.lgray_GC = XtGetGC((Widget)lw,
+			   (unsigned)GCForeground|GCBackground|GCFont|GCGraphicsExposures,
+			   &values);
+    }
 }
 
 static void compute_bitmap_offsets(clw, lw)
@@ -329,9 +610,9 @@ LabelWidget lw, clw;
      */
     if (lw->label.lbm_height != 0) {
 	lw->label.lbm_y = (((int)clw->core.height) -
-			   ((int) lw->label.internal_height * 2) -
-			   ((int) lw->label.lbm_height)) / 2;
-        lw->label.lbm_x = (int) lw->label.internal_width;
+			   ((int)lw->label.internal_height * 2) -
+			   ((int)lw->label.lbm_height)) / 2;
+        lw->label.lbm_x = (int)lw->label.internal_width;
     }
     else {
 	lw->label.lbm_y = 0;
@@ -396,10 +677,18 @@ static void ClassInitialize()
 static void Initialize(request, new)
  Widget request, new;
 {
-    LabelWidget lw = (LabelWidget) new;
+    LabelWidget lw = (LabelWidget)new;
 
-    if (lw->label.label == NULL) lw->label.label = XtNewString(lw->core.name);
-    else			 lw->label.label = XtNewString(lw->label.label);
+    if (lw->label.encoding)	/* Wide character encoding */
+    {
+      if (lw->label.label == NULL) lw->label.label = XtNewString(lw->core.name);
+      else lw->label.label = (char *)UTF8toUCS2(lw->label.label);
+    }
+    else	/* 8 bit character or multibyte character encoding */
+    {
+      if (lw->label.label == NULL) lw->label.label = XtNewString(lw->core.name);
+      else lw->label.label = XtNewString(lw->label.label);
+    }
 
     GetnormalGC(lw);
     GetgrayGCs(lw);
@@ -418,7 +707,6 @@ static void Initialize(request, new)
 
     lw->label.label_x = lw->label.label_y = 0;
     (*XtClass(new)->core_class.resize)(new);
-
 }
 
 /***********************************************************************
@@ -427,6 +715,26 @@ static void Initialize(request, new)
  *
  ***********************************************************************/
 
+static void _DrawLabel16(lw, labelText, y, len)
+LabelWidget lw;
+char16	   *labelText;
+int	    y, len;
+{
+  Display *disp = XtDisplay((Widget)lw);
+  Window   win  = XtWindow((Widget)lw);
+
+  if (XtIsSensitive((Widget)lw))
+    XDrawString16(disp, win, lw->label.normal_GC, lw->label.label_x, y,
+		  (XChar2b *)labelText, len);
+  else
+  {
+    XDrawString16(disp, win, lw->label.lgray_GC, lw->label.label_x+1, y+1,
+		  (XChar2b *)labelText, len);
+    XDrawString16(disp, win, lw->label.gray_GC, lw->label.label_x, y,
+		  (XChar2b *)labelText, len);
+  }
+}
+
 static void _DrawLabelString(lw, labelText, y, len)
 LabelWidget lw;
 char	   *labelText;
@@ -435,30 +743,31 @@ int	    y, len;
   Display *disp = XtDisplay((Widget)lw);
   Window   win  = XtWindow((Widget)lw);
 
-  if (lw->label.encoding) /* UNICODE string */
+  if (lw->simple.international == True)	 /* Multibyte string */
   {
     if (XtIsSensitive((Widget)lw))
-      XDrawString16(disp, win, lw->label.normal_GC, lw->label.label_x, y,
-		   (TXT16*)labelText, len/2);
+	XmbDrawString(disp, win, lw->label.fontset, lw->label.normal_GC, lw->label.label_x,
+		      y, labelText, len);
     else
     {
-	XDrawString16(disp, win, lw->label.lgray_GC, lw->label.label_x+1, y+1,
-		     (TXT16*)labelText, len/2);
-	XDrawString16(disp, win, lw->label.gray_GC, lw->label.label_x, y,
-		     (TXT16*)labelText, len/2);
+	XmbDrawString(disp, win, lw->label.fontset, lw->label.lgray_GC, lw->label.label_x,
+		      y, labelText, len);
+	XmbDrawString(disp, win, lw->label.fontset, lw->label.gray_GC, lw->label.label_x+1,
+		      y+1, labelText, len);
     }
   }
-  else /* ASCII character string */
+  else /* 8 Bit character string */
   {
     if (XtIsSensitive((Widget)lw))
-      XDrawString(disp, win, lw->label.normal_GC, lw->label.label_x, y, labelText, len);
+        XDrawString(disp, win, lw->label.normal_GC, lw->label.label_x, y, labelText, len);
     else
     {
-      XDrawString(disp, win, lw->label.lgray_GC, lw->label.label_x+1, y+1, labelText, len);
-      XDrawString(disp, win, lw->label.gray_GC, lw->label.label_x, y, labelText, len);
+        XDrawString(disp, win, lw->label.lgray_GC, lw->label.label_x+1, y+1, labelText, len);
+        XDrawString(disp, win, lw->label.gray_GC, lw->label.label_x, y, labelText, len);
     }
   }
 }
+
 /* ------------------------------------------------------------------ */
 
 static void Redisplay(w, event, region)
@@ -471,6 +780,7 @@ Region region;
    int		len = lw->label.label_len;
    char		*label = lw->label.label;
    Position	y = lw->label.label_y + lw->label.font->max_bounds.ascent;
+   XFontSetExtents *ext;
 
    if (lw->label.pixmap == None)
    {
@@ -478,7 +788,7 @@ Region region;
 
        if (lw->label.left_bitmap && lw->label.lbm_width != 0)
        {
-	 /* We only display the left bitmap/pixmap if it is in the region to redisplay. */
+	 /* We only draw the left bitmap/pixmap if it is in the region to redisplay. */
 
 	 if ((event == NULL) || (XRectInRegion(region, lw->label.lbm_x, lw->label.lbm_y,
 	      lw->label.lbm_width, lw->label.lbm_height) != RectangleOut))
@@ -503,24 +813,52 @@ Region region;
            }
 	 }
        }
-       /* Display label text if it is in the region to redisplay */
+       /* Draw label text if it is in the region to redisplay */
 
        if ((event == NULL) || (XRectInRegion(region, lw->label.label_x, lw->label.label_y,
 	    lw->label.label_width, lw->label.label_height) != RectangleOut))
        {
-         if (len == MULTI_LINE_LABEL) {
-	   char *nl;
-	   while ((nl = index(label, '\n')) != NULL)
+	 if (lw->label.encoding)	/* Label string with UCS2 code/UNICODE */
+	 {
+	    char16 *wnl, *wlabel = (char16 *)label;
+
+	    if (len == MULTI_LINE_LABEL)
+	    {
+	      while ((wnl = str16chr(wlabel, '\n')) != NULL)
+	      {
+	         len = (int)(wnl - wlabel);
+	         _DrawLabel16(lw, wlabel, (int)y, len);
+		 y += lw->label.font->max_bounds.ascent + lw->label.font->max_bounds.descent;
+	         wlabel = wnl + 1;
+	      }
+	      len = str16len(wlabel);
+            }
+            if (len) _DrawLabel16(lw, wlabel, (int)y, len);
+	 }
+	 else	/* Label string with 8 bit code or multibyte code */
+	 {
+	 /* Special handling of multibyte code: Get infos about the font set */
+
+	   if (lw->simple.international == True)
 	   {
-             len = lw->label.encoding ? (int)(nl - label)/2 : (int)(nl - label);
-	     _DrawLabelString(lw, label, (int)y, len);
-	     y += lw->label.font->max_bounds.ascent + lw->label.font->max_bounds.descent;
-	     label = nl + 1;
+	     ext = XExtentsOfFontSet(lw->label.fontset);
+	     y = lw->label.label_y + abs(ext->max_ink_extent.y);
 	   }
-	   len = strlen(label);
-         }
-         if (len) _DrawLabelString(lw, label, (int)y, len);
-       }
+ 	   if (len == MULTI_LINE_LABEL) {
+	     char *nl;
+	     while ((nl = index(label, '\n')) != NULL)
+	     {
+	       len = (int)(nl - label);
+	       _DrawLabelString(lw, label, (int)y, len);
+	       if (lw->simple.international == True) y += ext->max_ink_extent.height;
+	       else y += lw->label.font->max_bounds.ascent + lw->label.font->max_bounds.descent;
+	       label = nl + 1;
+	     }
+	     len = strlen(label);
+           }
+           if (len) _DrawLabelString(lw, label, (int)y, len);
+	 }
+      }
    }
    else	/* Label with only a bitmap or pixmap */
    {
@@ -549,10 +887,9 @@ Region region;
  *
  ***********************************************************************/
 
-/* DoTruncate(): Truncate the label string.
+/* DoTruncate8(): Truncate the 8 bit coded label string.
  * This function does not free the memory used by the label string.
  */
-
 static void DoTruncate8(lw, usableWidth)
 LabelWidget lw;
 int usableWidth;
@@ -563,7 +900,7 @@ int usableWidth;
    
    fullLen = strlen(lw->label.label_full_len);
 
-   /* Handling for a fixed width font: Calculate the string length */
+   /* Handling for a fixed width font: Calculate the string width in pixel */
 	 
    if (lw->label.font->max_bounds.width == lw->label.font->min_bounds.width)
      newlen = usableWidth / lw->label.font->max_bounds.width;
@@ -616,6 +953,144 @@ int usableWidth;
    }
 }
 
+/* DoTruncate16(): Truncate the label string in UNICODE/UCS2 code.
+ * This function does not free the memory used by the label string.
+ */
+
+static void DoTruncate16(lw, usableWidth)
+LabelWidget lw;
+int usableWidth;
+{
+   int newlen, fullLen, currentWidth;
+   static char *dots = "..";
+   char16 *p, *wLabel, *dots16;
+   
+   dots16 = UTF8toUCS2(dots);
+   fullLen = str16len((char16 *)lw->label.label_full_len);
+
+   /* Handling for a fixed width font: Calculate the string width in pixel */
+	 
+   if (lw->label.font->max_bounds.width == lw->label.font->min_bounds.width)
+     newlen = usableWidth / lw->label.font->max_bounds.width;
+   else
+   {
+     /* Handling for a proportional font:
+      * Calculate character by character how long the string have to be.
+      */
+     currentWidth = XTextWidth16(lw->label.font, (XChar2b *)dots16, 2);
+     newlen = 0;
+
+     if (lw->label.truncLeftSide) {	/* Truncate from the left side */
+	wLabel = (char16 *)lw->label.label_full_len;
+	p = &wLabel[fullLen];
+        while ((currentWidth < usableWidth) && (newlen < fullLen)) {
+           newlen++; p--;
+	   currentWidth += XTextWidth16(lw->label.font, (XChar2b *)p, 1);
+        }
+        newlen++;	/* for the dots, otherwise we have to decrement newlen */
+     }
+     else {		/* Truncate from the right side */
+	p = (char16 *)lw->label.label_full_len;
+        while ((currentWidth < usableWidth) && (newlen < fullLen)) {
+           newlen++;
+	   currentWidth += XTextWidth16(lw->label.font, (XChar2b *)p, 1);
+	   p++;
+        }
+        newlen++;	/* ... for the dots */
+     }
+   }
+   /* Check if our truncate mechanism works with the
+    * given label width. Otherwise we only copy two dots.
+    */
+   if (newlen > 3) {
+      lw->label.label = XtMalloc(sizeof(char16) * (newlen+1));
+      if (lw->label.truncLeftSide) {
+         str16cpy((char16 *)lw->label.label, dots16);
+	 wLabel = (char16 *)lw->label.label_full_len;
+         str16cat((char16 *)lw->label.label, &wLabel[fullLen - newlen + 2]);
+      }
+      else {
+	 wLabel = (char16 *)lw->label.label;
+         str16ncpy(wLabel, (char16 *)lw->label.label_full_len, newlen - 2);
+	 str16cat(wLabel, dots16);
+      }
+      lw->label.label_len = newlen;
+   }
+   else		/* The label is too small for the truncate mechanism */
+   {
+      lw->label.label = XtMalloc(3*sizeof(char16));
+      str16cpy((char16 *)lw->label.label, dots16);
+   }
+   XtFree((char *)dots16);
+}
+
+/* DoTruncateMulti(): Truncate the label string in multibyte code (UTF8).
+ * This function does not free the memory used by the label string.
+ */
+
+static void DoTruncateMulti(lw, usableWidth)
+LabelWidget lw;
+int usableWidth;
+{
+   int newlen, noOfChars, fullLen, currentWidth, charLen;
+   static char *dots = "..";		/* This is also a correct multibyte code */
+   char *p;
+   
+   fullLen = strlen(lw->label.label_full_len);
+
+   /* Handling for all fonts:
+    * Calculate character by character how long the string have to be.
+    */
+   newlen = noOfChars = 2;
+   currentWidth = XmbTextEscapement(lw->label.fontset, dots, noOfChars);
+
+   /* In international environments we always truncate from the right side */
+
+   p = lw->label.label_full_len;
+   while ((currentWidth < usableWidth) && (newlen < fullLen))
+   {
+	charLen = mblen(p, strlen(p));
+	currentWidth += XmbTextEscapement(lw->label.fontset, p, charLen);
+	noOfChars++;
+        newlen += charLen;
+	p += charLen;
+   }
+   /* Skip back one character because the last step was one step too much */
+
+   newlen -= charLen;
+   noOfChars--;
+
+   /* Check if our truncate mechanism works with the
+    * given label width. Otherwise we only copy two dots.
+    */
+   if (noOfChars > 3)
+   {
+      lw->label.label = XtMalloc(newlen+1);
+      strncpy(lw->label.label, lw->label.label_full_len, newlen - 2);
+      lw->label.label[newlen - 2] = '\0';
+      strcat(lw->label.label, dots);
+      lw->label.label_len = newlen;
+   }
+   else		/* The label is too small for the truncate mechanism */
+   {
+      lw->label.label = XtMalloc(3);
+      strcpy(lw->label.label, dots);
+   }
+}
+
+/* Free the resources allocated by the
+ * truncate mechanism and reset the label length
+ */
+static void ResetTruncateMode(lw)
+LabelWidget lw;
+{
+   XtFree(lw->label.label);
+   lw->label.label = lw->label.label_full_len;
+   lw->label.label_full_len = NULL;
+   if (lw->label.encoding) lw->label.label_len = str16len((char16 *)lw->label.label);
+   else lw->label.label_len = strlen(lw->label.label);
+}
+
 /* Check if the label string have to be truncated to display
  * it in the label. We don't do this, if we have a bitmap or
  * a multiline label.
@@ -629,7 +1104,7 @@ Dimension currentWidth;
    int usableWidth;
 
    if ((lw->label.pixmap == None) && (lw->label.label_len != MULTI_LINE_LABEL) &&
-       (*lw->label.label != '\0') && (lw->label.encoding == False))
+       (lw->label.label != NULL))
    {
       /* Label string will be displayed and is not multi line:
        * Check if the string fits into the widget.
@@ -643,17 +1118,17 @@ Dimension currentWidth;
 	  */
 	 if (lw->label.label_full_len == NULL) lw->label.label_full_len = lw->label.label;
 	 else XtFree(lw->label.label);
-	 DoTruncate8(lw, usableWidth);
+
+	 if (lw->label.encoding) DoTruncate16(lw, usableWidth);
+	 else
+	 {
+	    if (lw->simple.international) DoTruncateMulti(lw, usableWidth);
+	    else DoTruncate8(lw, usableWidth);
+	 }
       }
       else	/* After resize the label string fits into the widget */
       {
-	 if (lw->label.label_full_len != NULL)
-	 {
-	    XtFree(lw->label.label);
-	    lw->label.label = lw->label.label_full_len;
-	    lw->label.label_full_len = NULL;
-	    lw->label.label_len = strlen(lw->label.label);
-	 }
+	 if (lw->label.label_full_len != NULL) ResetTruncateMode(lw);
       }
    }
 }
@@ -674,8 +1149,8 @@ Dimension width, height;
     Position newPos;
     Position leftedge = lw->label.internal_width + LEFT_OFFSET(lw);
 
-    switch (lw->label.justify) {
-
+    switch (lw->label.justify)
+    {
 	case XtJustifyLeft   :
 	    newPos = leftedge;
 	    break;
@@ -686,13 +1161,12 @@ Dimension width, height;
 
 	case XtJustifyCenter :
 	default:
-	    newPos = (int)(width - lw->label.label_width) / 2;
+	    newPos = (Position)(width - lw->label.label_width /* + LEFT_OFFSET(lw) */ )/2;
 	    break;
     }
     if (newPos < (Position)leftedge) newPos = leftedge;
     lw->label.label_x = newPos;
     lw->label.label_y = (int)(height - lw->label.label_height)/2;
-    return;
 }
 
 static void Resize(w)
@@ -719,17 +1193,6 @@ static void Resize(w)
  *
  ***********************************************************************/
 
-/* Free the resources allocated by the
- * truncate mechanism
- */
-static void ResetTruncateMode(lw)
-LabelWidget lw;
-{
-   XtFree(lw->label.label);
-   lw->label.label = lw->label.label_full_len;
-   lw->label.label_full_len = NULL;
-}
-
 #define PIXMAP	 0
 #define WIDTH	 1
 #define HEIGHT	 2
@@ -760,17 +1223,23 @@ static Boolean SetValues(current, request, new, args, num_args)
     if (newlw->label.label == NULL) newlw->label.label = newlw->core.name;
 
     /*
-     * resize on bitmap change
+     * resize on bitmap change or change of font style
      */
     if ((curlw->label.left_bitmap != newlw->label.left_bitmap) ||
-	(curlw->label.left_clipMask != newlw->label.left_clipMask)) was_resized = True;
-    if (curlw->label.encoding != newlw->label.encoding) was_resized = True;
+	(curlw->label.left_clipMask != newlw->label.left_clipMask) ||
+        (curlw->label.encoding != newlw->label.encoding) ||
+        (curlw->simple.international && (curlw->label.fontset != newlw->label.fontset)))
+	  was_resized = True;
 
     if (curlw->label.label != newlw->label.label)
     {
        if (curlw->label.label != curlw->core.name) XtFree((char *)curlw->label.label);
        if (newlw->label.label != newlw->core.name)
-         newlw->label.label = XtNewString(newlw->label.label);
+       {
+	  if (newlw->label.encoding)
+	    newlw->label.label = (char *)UTF8toUCS2(newlw->label.label);
+          else newlw->label.label = XtNewString(newlw->label.label);
+       }
        if (newlw->label.label_full_len) XtFree(newlw->label.label_full_len);
        newlw->label.label_full_len = NULL;
        was_resized = True;
@@ -798,8 +1267,8 @@ static Boolean SetValues(current, request, new, args, num_args)
 
     if (curlw->label.foreground != newlw->label.foreground
 	|| curlw->core.background_pixel != newlw->core.background_pixel
-        || curlw->add3dExt.shadowColor != newlw->add3dExt.shadowColor
-        || curlw->add3dExt.highlightColor != newlw->add3dExt.highlightColor
+        || curlw->simple.shadowColor != newlw->simple.shadowColor
+        || curlw->simple.highlightColor != newlw->simple.highlightColor
 	|| curlw->label.font->fid != newlw->label.font->fid) {
 
 	XtReleaseGC(new, curlw->label.normal_GC);
@@ -813,7 +1282,7 @@ static Boolean SetValues(current, request, new, args, num_args)
     
     if (curlw->label.truncLabel != newlw->label.truncLabel)
     {
-	/* Truncate deactivated: If label is currently truncated
+	/* Truncate mode deactivated: If label is currently truncated,
 	 * release truncated string and reset the original label.
 	 */
         if (newlw->label.truncLabel == False)
@@ -821,7 +1290,6 @@ static Boolean SetValues(current, request, new, args, num_args)
     	  if (newlw->label.label_full_len)
 	  {
 	     ResetTruncateMode(newlw);
-	     newlw->label.label_len = strlen(newlw->label.label);
 	     redisplay = True;
 	  }
 	}
@@ -837,7 +1305,6 @@ static Boolean SetValues(current, request, new, args, num_args)
 	 * Recalculation of the string is required.
 	 */
 	ResetTruncateMode(newlw);
-	newlw->label.label_len = strlen(newlw->label.label);
 	was_resized = True;
     }
 
