@@ -1,39 +1,36 @@
-/* $XConsortium: List.c,v 1.34 91/09/27 18:35:07 converse Exp $ */
+/* $TOG: List.c /main/38 1998/02/11 14:53:23 kaleb $ */
 
 /*
- * Copyright 1989 Massachusetts Institute of Technology
- *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  M.I.T. makes no representations about the
- * suitability of this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
- *
- * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
- * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
- * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * 3D extensions added : Roland Krause 1996
- */
+Copyright 1989, 1994, 1998  The Open Group
+
+All Rights Reserved.
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of The Open Group shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from The Open Group.
+*/
 
 /*
  * List.c - List widget
  *
- * This is the List widget, it is useful to display a list, without the
- * overhead of having a widget for each item in the list.  It allows 
- * the user to select an item in a list and notifies the application through
- * a callback function.
+ * This is a List widget.  It allows the user to select an item in a list and
+ * notifies the application through a callback function.
  *
  *	Created: 	8/13/88
  *	By:		Chris D. Peterson
  *                      MIT X Consortium
+ *
+ * This file contains modifications for XawPlus, Roland Krause 2000
  */
 
 #include <stdio.h>
@@ -47,6 +44,16 @@
 #include <X11/XawPlus/XawInit.h>
 #include <X11/XawPlus/ListP.h>
 
+/* These added so widget knows whether its height, width are user selected.
+I also added the freedoms member of the list widget part. */
+
+#define HeightLock  1
+#define WidthLock   2
+#define LongestLock 4
+
+#define HeightFree( w )  !(((ListWidget)(w))->list.freedoms & HeightLock )
+#define WidthFree( w )   !(((ListWidget)(w))->list.freedoms & WidthLock )
+#define LongestFree( w ) !(((ListWidget)(w))->list.freedoms & LongestLock )
 
 /* 
  * Default Translation table.
@@ -73,8 +80,18 @@ static XtResource resources[] = {
        offset(simple.cursor), XtRString, "left_ptr"},
     {XtNfont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
 	offset(list.font),XtRString, XtDefaultFont},
+    {XtNfontSet,  XtCFontSet, XtRFontSet, sizeof(XFontSet ),
+	offset(list.fontset),XtRString, XtDefaultFontSet},
     {XtNlist, XtCList, XtRPointer, sizeof(char **),
        offset(list.list), XtRString, NULL},
+    {XtNiconList, XtCList, XtRPointer, sizeof(XawIconList *),
+       offset(list.iconList), XtRPointer, NULL},
+    {XtNiconWidth, XtCWidth, XtRDimension,  sizeof(Dimension),
+	offset(list.icon_width), XtRImmediate, (XtPointer)0},
+    {XtNiconHeight, XtCHeight, XtRDimension, sizeof(Dimension),
+	offset(list.icon_height), XtRImmediate, (XtPointer)0},
+    {XtNiconDepth, XtCDepth, XtRDimension, sizeof(Dimension),
+	offset(list.icon_depth), XtRImmediate, (XtPointer)1},
     {XtNdefaultColumns, XtCColumns, XtRInt,  sizeof(int),
 	offset(list.default_cols), XtRImmediate, (XtPointer)2},
     {XtNlongest, XtCLongest, XtRInt,  sizeof(int),
@@ -118,8 +135,7 @@ static XtActionsRec actions[] = {
 ListClassRec listClassRec = {
   {
 /* core_class fields */	
-#define superclass		(&simpleClassRec)
-    /* superclass	  	*/	(WidgetClass) superclass,
+    /* superclass	  	*/	(WidgetClass) &simpleClassRec,
     /* class_name	  	*/	"List",
     /* widget_size	  	*/	sizeof(ListRec),
     /* class_initialize   	*/	XawInitializeWidgetSet,
@@ -152,17 +168,18 @@ ListClassRec listClassRec = {
     /* display_accelerator	*/	NULL,
     /* extension		*/	NULL
   },
-/* Add3dExt fields initialization */
-  {
-    /* unused			*/	0
-  },
 /* Simple class fields initialization */
   {
     /* change_sensitive		*/	XtInheritChangeSensitive
-  }
+  },
+/* List class fields initialization */
+  {
+    /* not used			*/	0
+  },
 };
 
 WidgetClass listWidgetClass = (WidgetClass)&listClassRec;
+WidgetClass iconListWidgetClass = (WidgetClass)&listClassRec;
 
 /****************************************************************
  *
@@ -176,66 +193,129 @@ Widget w;
     XGCValues	values;
     ListWidget lw = (ListWidget) w;    
 
-    values.foreground	= lw->list.foreground;
-    values.font		= lw->list.font->fid;
-    lw->list.normgc = XtGetGC(w, (unsigned) GCForeground | GCFont,
-				 &values);
+    values.foreground = lw->list.foreground;
+    values.background = lw->core.background_pixel;
+    values.font	      = lw->list.font->fid;
 
-    values.foreground	= lw->core.background_pixel;
-    lw->list.revgc = XtGetGC(w, (unsigned) GCForeground | GCFont,
-				 &values);
+    if (lw->simple.international == True)
+    {
+      lw->list.normgc = XtAllocateGC(w, 0, GCForeground|GCBackground, &values, GCFont, 0);
+      lw->list.iconGc = XtAllocateGC(w, 0, GCForeground|GCBackground, &values, 0, 0);
+    }
+    else
+    {
+      lw->list.normgc = XtGetGC(w, GCForeground|GCBackground|GCFont, &values);
+      lw->list.iconGc = XtGetGC(w, GCForeground|GCBackground, &values);
+    }
 
-    values.tile       = XmuCreateStippledPixmap(XtScreen(w), 
-						lw->list.foreground,
-						lw->core.background_pixel,
-						lw->core.depth);
+    values.foreground = lw->core.background_pixel;
+    values.background = lw->list.foreground;
+
+    if (lw->simple.international == True)
+      lw->list.revgc = XtAllocateGC(w, 0, GCForeground|GCBackground, &values, GCFont, 0);
+    else lw->list.revgc = XtGetGC(w, GCForeground|GCFont|GCBackground, &values);
+
+    values.tile = XmuCreateStippledPixmap(XtScreen(w), 
+					  lw->list.foreground,
+					  lw->core.background_pixel,
+					  lw->core.depth);
     values.fill_style = FillTiled;
 
-    lw->list.graygc = XtGetGC(w, (unsigned) GCFont | GCTile | GCFillStyle,
-			      &values);
+    if (lw->simple.international == True)
+      lw->list.graygc = XtAllocateGC(w, 0, GCTile|GCFillStyle, &values, GCFont, 0);
+    else lw->list.graygc = XtGetGC( w, GCFont|GCTile|GCFillStyle, &values);
+}
+
+/* CalculatedValues()
+ *
+ * does routine checks/computations that must be done after data changes
+ * but won't hurt if accidently called
+ *
+ * These calculations were needed in SetValues.  They were in ResetList.
+ * ResetList called ChangeSize, which made an XtGeometryRequest.  You
+ * MAY NOT change your geometry from within a SetValues. (Xt man,
+ * sect. 9.7.2)  So, I factored these changes out. */
+
+static void CalculatedValues( w )
+Widget w;
+{
+    int i, len;
+
+    ListWidget lw = (ListWidget) w;
+
+    /* If both lists are NULL then the list will just be the name of the widget. */
+
+    if ((lw->list.list == NULL) && (lw->list.iconList == NULL)) {
+      lw->list.list = &(lw->core.name);
+      lw->list.nitems = 1;
+    }
+
+    /* Get the number of items. One list is always defined here ! */
+
+    if (lw->list.iconList)
+    {
+     /* If an icon list is available we use it with a
+      * higher priority as the text list.
+      */
+      if (lw->list.nitems == 0)
+        for ( ; lw->list.iconList[lw->list.nitems].string != NULL; lw->list.nitems++);
+    }
+    else if ((lw->list.list) && (lw->list.nitems == 0))
+        for ( ; lw->list.list[lw->list.nitems] != NULL; lw->list.nitems++);
+
+    /* Get column width. */
+
+    if (LongestFree(lw))
+    {
+        lw->list.longest = 0; /* so it will accumulate real longest below */
+
+	if (lw->list.iconList)
+	{
+          for ( i = 0 ; i < lw->list.nitems; i++)
+	  {
+            if (lw->simple.international == True)
+	      len = XmbTextEscapement(lw->list.fontset, lw->list.iconList[i].string,
+				      strlen(lw->list.iconList[i].string));
+            else len = XTextWidth(lw->list.font, lw->list.iconList[i].string,
+				  strlen(lw->list.iconList[i].string));
+            if (len > lw->list.longest) lw->list.longest = len;
+          }
+	}
+	else
+	{
+          for ( i = 0 ; i < lw->list.nitems; i++)
+	  {
+            if (lw->simple.international == True)
+	      len = XmbTextEscapement(lw->list.fontset, lw->list.list[i], strlen(lw->list.list[i]));
+            else len = XTextWidth(lw->list.font, lw->list.list[i], strlen(lw->list.list[i]));
+            if (len > lw->list.longest) lw->list.longest = len;
+          }
+	}
+    }
+    lw->list.col_width = lw->list.longest + lw->list.column_space;
+    if (lw->list.iconList) lw->list.col_width += lw->list.icon_width;
 }
 
 /*	Function Name: ResetList
  *	Description: Resets the new list when important things change.
  *	Arguments: w - the widget.
  *                 changex, changey - allow the height or width to change?
- *	Returns: none.
+ *
+ *	Returns: TRUE if width or height have been changed
  */
 
 static void
-ResetList(w, changex, changey)
+ResetList( w, changex, changey )
 Widget w;
 Boolean changex, changey;
 {
-    ListWidget lw = (ListWidget) w;
     Dimension width = w->core.width;
     Dimension height = w->core.height;
-    register int i, len;
 
-/*
- * If list is NULL then the list will just be the name of the widget.
- */
+    CalculatedValues( w );
 
-    if (lw->list.list == NULL) {
-      lw->list.list = &(lw->core.name);
-      lw->list.nitems = 1;
-    }
-
-    if (lw->list.nitems == 0)	    /* Get number of items. */
-        for ( ; lw->list.list[lw->list.nitems] != NULL ; lw->list.nitems++);
-
-    if (lw->list.longest == 0) /* Get column width. */
-        for ( i = 0 ; i < lw->list.nitems; i++) {
-	    len = XTextWidth(lw->list.font, lw->list.list[i],
-			     strlen(lw->list.list[i]));
-	    if (len > lw->list.longest)
-	        lw->list.longest = len;
-	}
-
-    lw->list.col_width = lw->list.longest + lw->list.column_space;
-
-    if (Layout(w, changex, changey, &width, &height))
-      ChangeSize(w, width, height);
+    if( Layout( w, changex, changey, &width, &height ) )
+      ChangeSize( w, width, height );
 }
 
 /*	Function Name: ChangeSize.
@@ -254,9 +334,10 @@ Dimension width, height;
     request.request_mode = CWWidth | CWHeight;
     request.width = width;
     request.height = height;
-    
+
     switch ( XtMakeGeometryRequest(w, &request, &reply) ) {
     case XtGeometryYes:
+        break;
     case XtGeometryNo:
         break;
     case XtGeometryAlmost:
@@ -270,14 +351,7 @@ Dimension width, height;
 	    break;
 	case XtGeometryAlmost:
 	    request = reply;
-	    if (Layout(w, FALSE, FALSE,
-		       &(request.width), &(request.height))) {
-	      char buf[BUFSIZ];
-	      sprintf(buf, "List Widget: %s %s",
-		      "Size Changed when it shouldn't have",
-		      "when computing layout");
-	      XtAppWarning(XtWidgetToApplicationContext(w), buf);
-	    }
+	    Layout(w, FALSE, FALSE, &(request.width), &(request.height));
 	    request.request_mode = CWWidth | CWHeight;
 	    XtMakeGeometryRequest(w, &request, &reply);
 	    break;
@@ -312,15 +386,24 @@ Cardinal *num_args;
  * Initialize all private resources.
  */
 
+    /* record for posterity if we are free */
+    lw->list.freedoms = (lw->core.width != 0) * WidthLock +
+                        (lw->core.height != 0) * HeightLock +
+                        (lw->list.longest != 0) * LongestLock;
+
     GetGCs(new);
 
-    /* Set row height. */
-    lw->list.row_height = lw->list.font->max_bounds.ascent
-			+ lw->list.font->max_bounds.descent
-			+ lw->list.row_space;
+    /* Set row height. based on font or fontset */
 
-    ResetList(new, (new->core.width == 0), (new->core.height == 0));
+    if (lw->simple.international == True)
+      lw->list.row_height = XExtentsOfFontSet(lw->list.fontset)->max_ink_extent.height;
+    else lw->list.row_height = lw->list.font->max_bounds.ascent + lw->list.font->max_bounds.descent;
 
+    if (lw->list.iconList && (lw->list.row_height < lw->list.icon_height))
+	lw->list.row_height = lw->list.icon_height + lw->list.row_space;
+    else lw->list.row_height += lw->list.row_space;
+
+    ResetList(new, WidthFree(lw), HeightFree(lw));
     lw->list.highlight = lw->list.is_highlighted = NO_HIGHLIGHT;
 
 } /* Initialize */
@@ -411,7 +494,7 @@ Widget w;
 int ul, lr, item;
 {
     ListWidget lw = (ListWidget) w;
-    register int mod_item;
+    int mod_item;
     int things;
     
     if (item < ul || item > lr) 
@@ -431,36 +514,33 @@ int ul, lr, item;
  *	Description: paints the color of the background for the given item.
  *	Arguments: w - the widget.
  *                 x, y - ul corner of the area item occupies.
- *                 item - the item we are dealing with.
  *                 gc - the gc that is used to paint this rectangle
  *	Returns: 
  */
 
 static void
-HighlightBackground(w, x, y, item, gc)
+HighlightBackground(w, x, y, gc)
 Widget w;
-int x, y, item;
+int x, y;
 GC gc;
 {
     ListWidget lw = (ListWidget) w;
-    int hl_x, hl_y, width, height;
+    int hl_x, hl_y, width;
 
-    hl_x = x - lw->list.column_space/2;
-    width  = lw->list.col_width;
-    hl_y = y - lw->list.row_space/2;
-    height = lw->list.row_height + lw->list.row_space;
+    hl_x   = x - lw->list.column_space/2;
+    width  = lw->list.col_width + lw->list.column_space/2;
+    hl_y   = y - lw->list.row_space/2;
 
-    XFillRectangle(XtDisplay(w), XtWindow(w), gc, hl_x, hl_y, width, height);
+    XFillRectangle(XtDisplay(w), XtWindow(w), gc, hl_x, hl_y, width, lw->list.row_height);
 }
 
-/*	Function Name: PaintItemName
- *	Description: paints the name of the item in the appropriate location.
- *	Arguments: w - the list widget.
- *                 item - the item to draw.
- *	Returns: none.
+/*  PaintItemName()
  *
- *      NOTE: no action taken on an unrealized widget.
- */
+ *  paints the name of the item in the appropriate location.
+ *  w - the list widget.
+ *  item - the item to draw.
+ *
+ *  NOTE: no action taken on an unrealized widget. */
 
 static void
 PaintItemName(w, item)
@@ -469,65 +549,116 @@ int item;
 {
     char * str;
     GC gc;
-    int x, y, str_y;
+    int x, y, str_x, str_y, h_x, delta;
     ListWidget lw = (ListWidget) w;
+    XFontSetExtents *ext  = XExtentsOfFontSet(lw->list.fontset);
 
     if (!XtIsRealized(w)) return; /* Just in case... */
-   
+
     if (lw->list.vertical_cols) {
-	x = lw->list.col_width * (item / lw->list.nrows)
-	  + lw->list.internal_width;
-        y = lw->list.row_height * (item % lw->list.nrows)
-	  + lw->list.internal_height;
+	x = (lw->list.col_width + lw->list.internal_width) * (item / lw->list.nrows);
+        y = lw->list.row_height * (item % lw->list.nrows) + lw->list.internal_height;
     }
     else {
-        x = lw->list.col_width * (item % lw->list.ncols)
-	  + lw->list.internal_width;
-        y = lw->list.row_height * (item / lw->list.ncols)
-	  + lw->list.internal_height;
+        x = (lw->list.col_width + lw->list.internal_width) * (item % lw->list.ncols);
+        y = lw->list.row_height * (item / lw->list.ncols) + lw->list.internal_height;
     }
+    /* Determine the string and the icon position */
 
-    str_y = y + lw->list.font->max_bounds.ascent;
+    if (lw->list.iconList)
+      str_x  = x + lw->list.icon_width + lw->list.internal_width;
+    else str_x = x;
+    h_x = x + lw->list.internal_width;
+
+    if (lw->simple.international) {
+	delta = (lw->list.row_height - ext->max_ink_extent.height)/2 - lw->list.internal_height;
+	str_y = y + abs(ext->max_ink_extent.y) + delta; 
+    }
+    else {
+	delta = (lw->list.row_height - lw->list.font->max_bounds.ascent -
+		 lw->list.font->max_bounds.descent)/2 - lw->list.internal_height;
+	str_y = y + lw->list.font->max_bounds.ascent + delta;
+    }
+    /* Paint or erase the highlighted background */
 
     if (item == lw->list.is_highlighted) {
         if (item == lw->list.highlight) {
             gc = lw->list.revgc;
-	    HighlightBackground(w, x, y, item, lw->list.normgc);
+	    HighlightBackground(w, h_x, y, lw->list.normgc);
 	}
         else {
-	    if (XtIsSensitive(w)) 
-	        gc = lw->list.normgc;
-	    else
-	        gc = lw->list.graygc;
-	    HighlightBackground(w, x, y, item, lw->list.revgc);
+	    if (XtIsSensitive(w)) gc = lw->list.normgc;
+	    else gc = lw->list.graygc;
+	    HighlightBackground(w, h_x, y, lw->list.revgc);
 	    lw->list.is_highlighted = NO_HIGHLIGHT;
         }
     }
     else {
         if (item == lw->list.highlight) {
             gc = lw->list.revgc;
-	    HighlightBackground(w, x, y, item, lw->list.normgc);
+	    HighlightBackground(w, h_x, y, lw->list.normgc);
 	    lw->list.is_highlighted = item;
 	}
 	else {
-	    if (XtIsSensitive(w)) 
-	        gc = lw->list.normgc;
-	    else
-	        gc = lw->list.graygc;
+	    if (XtIsSensitive(w)) gc = lw->list.normgc;
+	    else gc = lw->list.graygc;
 	}
     }
+    /*	Paint the icon if there is one. The icon will only be painted, if
+ 	width and height are greater than 0 and if an icon is defined for
+	this entry. The depth of the icon has to be 1 (normal bitmap)
+	or of the same color depth as your display */
 
-    str =  lw->list.list[item];	/* draw it */
-    XDrawString(XtDisplay(w), XtWindow(w), gc, x, str_y, str, strlen(str));
+    if ((lw->list.iconList) &&
+	(lw->list.icon_width > 0) && (lw->list.icon_height > 0)) {
+      x += lw->list.column_space/2;
+      if (lw->list.iconList[item].bitmap != XtUnspecifiedPixmap)
+	if (lw->list.icon_depth == 1)
+          XCopyPlane(XtDisplay(w),lw->list.iconList[item].bitmap,XtWindow(w),
+		lw->list.normgc,0,0,lw->list.icon_width,lw->list.icon_height, x, y, 1L);
+	else
+	  if (lw->list.iconList[item].clipMask != XtUnspecifiedPixmap)
+          {
+	    /* Support of a clip mask for non rectangular or transparent
+	     * pixmaps: Manipulate the GC with a clip mask
+	     */
+	     XSetClipMask(XtDisplay(w),lw->list.iconGc,lw->list.iconList[item].clipMask);
+	     XSetClipOrigin(XtDisplay(w),lw->list.iconGc, x, y);
+             XCopyArea(XtDisplay(w),lw->list.iconList[item].bitmap,XtWindow(w),
+		 lw->list.iconGc,0,0,lw->list.icon_width,lw->list.icon_height, x, y);
+          }
+	  else
+            XCopyArea(XtDisplay(w),lw->list.iconList[item].bitmap,XtWindow(w),
+		 lw->list.normgc,0,0,lw->list.icon_width,lw->list.icon_height, x, y);
+      else XClearArea(XtDisplay(w), XtWindow(w), x, y,
+		 lw->list.icon_width,lw->list.icon_height,FALSE);
+    }
+    /* List's overall width contains the same number of inter-column
+       column_space's as columns.  There should thus be a half
+       column_width margin on each side of each column.
+       The row case is symmetric. */
+
+    str_x  += lw->list.column_space/2;
+    str_y  += lw->list.row_space/2;
+
+    /* Draw the String */
+
+    if (lw->list.iconList) str = lw->list.iconList[item].string;
+    else str = lw->list.list[item];
+
+    if (lw->simple.international)
+        XmbDrawString(XtDisplay(w), XtWindow(w), lw->list.fontset,
+		      gc, str_x, str_y, str, strlen(str));
+    else XDrawString(XtDisplay(w), XtWindow(w), gc, str_x, str_y, str, strlen(str));
 }
+
     
-/*	Function Name: Redisplay
- *	Description: Repaints the widget window on expose events.
- *	Arguments: w - the list widget.
- *                 event - the expose event for this repaint.
- *                 junk - NOT USED.
- *	Returns: 
- */
+/* Redisplay()
+ *
+ * Repaints the widget window on expose events.
+ * w - the list widget.
+ * event - the expose event for this repaint.
+ * junk - not used, unless three-d patch enabled. */
 
 /* ARGSUSED */
 static void 
@@ -553,14 +684,14 @@ Region junk;
 	PaintItemName(w, item);
 }
 
-/*	Function Name: PreferredGeom
- *	Description: This tells the parent what size we would like to be
- *                   given certain constraints.
- *	Arguments: w - the widget.
- *                 intended - what the parent intends to do with us.
- *                 requested - what we want to happen.
- *	Returns: none.
- */
+
+/* PreferredGeom()
+ *
+ * This tells the parent what size we would like to be
+ * given certain constraints.
+ * w - the widget.
+ * intended - what the parent intends to do with us.
+ * requested - what we want to happen. */
 
 static XtGeometryResult 
 PreferredGeom(w, intended, requested)
@@ -604,38 +735,37 @@ XtWidgetGeometry *intended, *requested;
     return(XtGeometryYes);
 }
 
-/*	Function Name: Resize
- *	Description: resizes the widget, by changing the number of rows and
- *                   columns.
- *	Arguments: w - the widget.
- *	Returns: none.
- */
+
+/* Resize()
+ *
+ * resizes the widget, by changing the number of rows and columns. */
 
 static void
 Resize(w)
-Widget w;
+    Widget w;
 {
-  Dimension width, height;
+    Dimension width, height;
 
-  width = w->core.width;
-  height = w->core.height;
+    width = w->core.width;
+    height = w->core.height;
 
-  if (Layout(w, FALSE, FALSE, &width, &height))
-    XtAppWarning(XtWidgetToApplicationContext(w),
-	    "List Widget: Size changed when it shouldn't have when resising.");
+    if (Layout(w, FALSE, FALSE, &width, &height))
+	XtAppWarning(XtWidgetToApplicationContext(w),
+	   "List Widget: Size changed when it shouldn't have when resising.");
 }
 
-/*	Function Name: Layout
- *	Description: lays out the item in the list.
- *	Arguments: w - the widget.
- *                 xfree, yfree - TRUE if we are free to resize the widget in
- *                                this direction.
- *                 width, height - the is the current width and height that 
- *                                 we are going to layout the list widget to,
- *                                 depending on xfree and yfree of course.
+
+/* Layout()
+ *
+ * lays out the item in the list.
+ * w - the widget.
+ * xfree, yfree - TRUE if we are free to resize the widget in
+ *                this direction.
+ * width, height- the is the current width and height that we are going
+ *                we are going to layout the list widget to,
+ *                depending on xfree and yfree of course.
  *                               
- *	Returns: TRUE if width or height have been changed.
- */
+ * RETURNS: TRUE if width or height have been changed. */
 
 static Boolean
 Layout(w, xfree, yfree, width, height)
@@ -657,7 +787,12 @@ Dimension *width, *height;
 	/* 12/3 = 4 and 10/3 = 4, but 9/3 = 3 */
 	lw->list.nrows = ( ( lw->list.nitems - 1) / lw->list.ncols) + 1 ;
 	if (xfree) {		/* If allowed resize width. */
-	    *width = lw->list.ncols * lw->list.col_width 
+
+            /* this counts the same number
+            of inter-column column_space 's as columns.  There should thus be a
+            half column_space margin on each side of each column...*/
+
+	    *width = lw->list.ncols * lw->list.col_width
 	           + 2 * lw->list.internal_width;
 	    change = TRUE;
 	}
@@ -718,16 +853,12 @@ Dimension *width, *height;
     return(change);
 }
 
-/*	Function Name: Notify
- *	Description: Notifies the user that a button has been pressed, and
- *                   calles the callback, if the XtNpasteBuffer resource
- *                   is true then the name of the item is also put in the
- *                   X cut buffer ( buf (0) ).
- *	Arguments: w - the widget that the notify occured in.
- *                 event - event that caused this notification.
- *                 params, num_params - not used.
- *	Returns: none.
- */
+
+/* Notify() - ACTION
+ *
+ * Notifies the user that a button has been pressed, and
+ * calls the callback; if the XtNpasteBuffer resource is true
+ * then the name of the item is also put in CUT_BUFFER0.	*/
 
 /* ARGSUSED */
 static void
@@ -737,8 +868,9 @@ XEvent * event;
 String * params;
 Cardinal *num_params;
 {
-    ListWidget lw = ( ListWidget ) w;
-    int item, item_len;
+    ListWidget lw = (ListWidget) w;
+    int item;
+    char *str;
     XawListReturnStruct ret_value;
 
 /* 
@@ -748,34 +880,27 @@ Cardinal *num_params;
  * notify, so unhighlight and return.
  */
 
-    if ( ((CvtToItem(w, event->xbutton.x, event->xbutton.y, &item))
-	  == OUT_OF_RANGE) || (lw->list.highlight != item) ) {
+    if (((CvtToItem(w, event->xbutton.x, event->xbutton.y, &item)) == OUT_OF_RANGE) ||
+	(lw->list.highlight != item)) {
         XawListUnhighlight(w);
         return;
     }
+    /* if XtNpasteBuffer is set then paste it. */
 
-    item_len = strlen(lw->list.list[item]);
+    str = lw->list.iconList ? lw->list.iconList[item].string : lw->list.list[item];
+    if (lw->list.paste) XStoreBytes(XtDisplay(w), str, strlen(str));
 
-    if ( lw->list.paste )	/* if XtNpasteBuffer set then paste it. */
-        XStoreBytes(XtDisplay(w), lw->list.list[item], item_len);
+    /* Call Callback function. */
 
-/* 
- * Call Callback function.
- */
-
-    ret_value.string = lw->list.list[item];
+    ret_value.string = str;
     ret_value.list_index = item;
-    
     XtCallCallbacks( w, XtNcallback, (XtPointer) &ret_value);
 }
 
-/*	Function Name: Unset
- *	Description: unhighlights the current element.
- *	Arguments: w - the widget that the event occured in.
- *                 event - not used.
- *                 params, num_params - not used.
- *	Returns: none.
- */
+
+/* Unset() - ACTION
+ *
+ * unhighlights the current element. */
 
 /* ARGSUSED */
 static void
@@ -788,13 +913,10 @@ Cardinal *num_params;
   XawListUnhighlight(w);
 }
 
-/*	Function Name: Set
- *	Description: Highlights the current element.
- *	Arguments: w - the widget that the event occured in.
- *                 event - event that caused this notification.
- *                 params, num_params - not used.
- *	Returns: none.
- */
+
+/* Set() - ACTION
+ *
+ * Highlights the current element. */
 
 /* ARGSUSED */
 static void
@@ -818,7 +940,8 @@ Cardinal *num_params;
  * Set specified arguments into widget
  */
 
-static Boolean SetValues(current, request, new, args, num_args)
+static Boolean 
+SetValues(current, request, new, args, num_args)
 Widget current, request, new;
 ArgList args;
 Cardinal *num_args;
@@ -827,10 +950,26 @@ Cardinal *num_args;
     ListWidget rl = (ListWidget) request;
     ListWidget nl = (ListWidget) new;
     Boolean redraw = FALSE;
+    int	text_height;
+    XFontSetExtents *ext = XExtentsOfFontSet(nl->list.fontset);
 
-    if ((cl->list.foreground != rl->list.foreground) ||
-	(cl->core.background_pixel != rl->core.background_pixel) ||
-	(cl->list.font != rl->list.font) ) {
+    /* If the request height/width is different, lock it.  Unless its 0. If */
+    /* neither new nor 0, leave it as it was.  Not in R5. */
+
+    if (nl->core.width != cl->core.width) nl->list.freedoms |= WidthLock;
+    if (nl->core.width == 0) nl->list.freedoms &= ~WidthLock;
+
+    if (nl->core.height != cl->core.height) nl->list.freedoms |= HeightLock;
+    if (nl->core.height == 0) nl->list.freedoms &= ~HeightLock;
+
+    if (nl->list.longest != cl->list.longest) nl->list.freedoms |= LongestLock;
+    if (nl->list.longest == 0) nl->list.freedoms &= ~LongestLock;
+
+    /* _DONT_ check for fontset here - it's not in GC.*/
+
+    if (  (cl->list.foreground       != nl->list.foreground)       ||
+	  (cl->core.background_pixel != nl->core.background_pixel) ||
+	  (cl->list.font             != nl->list.font)                ) {
 	XGCValues values;
 	XGetGCValues(XtDisplay(current), cl->list.graygc, GCTile, &values);
 	XmuReleaseStippledPixmap(XtScreen(current), values.tile);
@@ -840,46 +979,68 @@ Cardinal *num_args;
         GetGCs(new);
         redraw = TRUE;
     }
-
     /* Reset row height. */
 
-    if ((cl->list.row_space != rl->list.row_space) ||
-	(cl->list.font != rl->list.font)) 
-        nl->list.row_height = nl->list.font->max_bounds.ascent
-	                    + nl->list.font->max_bounds.descent
-			    + nl->list.row_space;
-    
-    if ((cl->core.width != rl->core.width)                     ||
-	(cl->core.height != rl->core.height)                   ||
-	(cl->list.internal_width != rl->list.internal_width)   ||
-	(cl->list.internal_height != rl->list.internal_height) ||
-	(cl->list.column_space != rl->list.column_space)       ||
-	(cl->list.row_space != rl->list.row_space)             ||
-	(cl->list.default_cols != rl->list.default_cols)       ||
-	(  (cl->list.force_cols != rl->list.force_cols) &&
-	   (rl->list.force_cols != rl->list.ncols) )           ||
-	(cl->list.vertical_cols != rl->list.vertical_cols)     ||
-	(cl->list.longest != rl->list.longest)                 ||
-	(cl->list.nitems != rl->list.nitems)                   ||
-	(cl->list.font != rl->list.font)                       ||
-	(cl->list.list != rl->list.list)                        ) {
+    if ((cl->list.font != nl->list.font) && (cl->simple.international == False))
+    {
+      /* Check, if the row height is determined with the
+	 font or with the icon and set it */
 
-      ResetList(new, TRUE, TRUE);
-      redraw = TRUE;
+      text_height = nl->list.font->max_bounds.ascent + nl->list.font->max_bounds.descent;
+      if (text_height > nl->list.icon_height)
+        nl->list.row_height = text_height + nl->list.row_space;
+      else nl->list.row_height = nl->list.icon_height + nl->list.row_space;
+    }
+    else if ((cl->list.fontset != nl->list.fontset) && (cl->simple.international == True))
+    {
+       if (ext->max_ink_extent.height > nl->list.icon_height)
+         nl->list.row_height = ext->max_ink_extent.height + nl->list.row_space;
+       else nl->list.row_height = nl->list.icon_height + nl->list.row_space;
+    }
+    /* ...If the above two font(set) change checkers above both failed, check
+    if row_space was altered.  If one of the above passed, row_height will
+    already have been re-calculated. */
+
+    else if (cl->list.row_space != nl->list.row_space)
+      nl->list.row_height = cl->list.row_height + nl->list.row_space - cl->list.row_space;
+
+    if ((cl->core.width           != nl->core.width) ||
+	(cl->core.height          != nl->core.height) ||
+	(cl->list.iconList        != nl->list.iconList) ||
+	((cl->list.list           != nl->list.list) && (nl->list.iconList == FALSE)) ||
+	((cl->list.icon_height	  != nl->list.icon_height) && nl->list.iconList) ||
+	((cl->list.icon_width	  != nl->list.icon_width)  && nl->list.iconList) ||
+	(cl->list.internal_width  != nl->list.internal_width) ||
+	(cl->list.internal_height != nl->list.internal_height) ||
+	(cl->list.column_space    != nl->list.column_space) ||
+	(cl->list.row_space       != nl->list.row_space) ||
+	(cl->list.default_cols    != nl->list.default_cols) ||
+	((cl->list.force_cols     != nl->list.force_cols) && (rl->list.force_cols != nl->list.ncols)) ||
+	(cl->list.vertical_cols   != nl->list.vertical_cols) ||
+	(cl->list.longest         != nl->list.longest) ||
+	(cl->list.nitems          != nl->list.nitems) ||
+	((cl->list.font           != nl->list.font) && (nl->simple.international == FALSE)) ||
+
+   /* Equiv. fontsets might have different values, but the same fonts, so the
+      next comparison is sloppy but not dangerous.  */
+
+	((cl->list.fontset        != nl->list.fontset) && nl->simple.international))
+    {
+        CalculatedValues(new);
+        Layout(new, WidthFree(nl), HeightFree(nl), &nl->core.width, &nl->core.height);
+        redraw = TRUE;
     }
 
-    if (cl->list.list != rl->list.list)
+    if ((cl->list.list != nl->list.list) || (cl->list.iconList != nl->list.iconList))
 	nl->list.is_highlighted = nl->list.highlight = NO_HIGHLIGHT;
 
-    if ((cl->core.sensitive != rl->core.sensitive) ||
-	(cl->core.ancestor_sensitive != rl->core.ancestor_sensitive)) {
+    if ((cl->core.sensitive != nl->core.sensitive) ||
+	(cl->core.ancestor_sensitive != nl->core.ancestor_sensitive)) {
         nl->list.highlight = NO_HIGHLIGHT;
 	redraw = TRUE;
     }
     
-    if (!XtIsRealized(current))
-      return(FALSE);
-      
+    if (!XtIsRealized(current)) return(FALSE);
     return(redraw);
 }
 
@@ -894,6 +1055,7 @@ static void Destroy(w)
     XtReleaseGC(w, lw->list.graygc);
     XtReleaseGC(w, lw->list.revgc);
     XtReleaseGC(w, lw->list.normgc);
+    XtReleaseGC(w, lw->list.iconGc);
 }
 
 /* Exported Functions */
@@ -912,35 +1074,97 @@ static void Destroy(w)
  *                 If nitems is <= 0 then the list needs to be NULL terminated.
  */
 
-void
-#if NeedFunctionPrototypes
-XawListChange(Widget w, char ** list, int nitems, int longest,
-#if NeedWidePrototypes
-	      int resize_it)
-#else
-	      Boolean resize_it)
-#endif
-#else
-XawListChange(w, list, nitems, longest, resize_it)
-Widget w;
-char ** list;
-int nitems, longest;
-Boolean resize_it;
-#endif
+void XawListChange(w, list, nitems, longest, resize_it)
+Widget	w;
+char  **list;
+int	nitems, longest;
+Boolean	resize_it;
 {
     ListWidget lw = (ListWidget) w;
 
-    lw->list.list = list;
+    if (nitems < 0) nitems = 0;
+    if (longest < 0) longest = 0;
 
-    if (nitems <= 0) nitems = 0;
-    lw->list.nitems = nitems;
-    if (longest <= 0) longest = 0;
-    lw->list.longest = longest;
+    lw->list.list     = list;
+    lw->list.iconList = NULL;	/* To avoid the use of an iconList */
+    lw->list.nitems   = nitems;
+    lw->list.longest  = longest;
 
-    ResetList(w, resize_it, resize_it);
+    /* If the user passes 0 meaning "calculate it", it must be free */
+
+    if (longest != 0) lw->list.freedoms |= LongestLock;
+    else lw->list.freedoms &= ~LongestLock;
+
+    /* else - still resize if its not locked */
+
+    if (resize_it) lw->list.freedoms &= ~WidthLock & ~HeightLock;
+
+    /* Redisplay the list */
+
+    ResetList(w, WidthFree(w), HeightFree(w));
     lw->list.is_highlighted = lw->list.highlight = NO_HIGHLIGHT;
-    if ( XtIsRealized(w) )
-      Redisplay(w, NULL, NULL);
+    if (XtIsRealized(w)) Redisplay(w, (XEvent *)NULL, (Region)NULL);
+}
+
+/*	Function Name: XawIconListChange.
+ *	Description: Changes the ICONLIST being used and shown.
+ *	Arguments: w - 		the list widget.
+ *                 iconlist - 	the new list.
+ *                 nitems - 	the number of items in the list.
+ *                 longest - 	the length (in Pixels) of the longest element
+ *                           	in the list.
+ *		   icon_w -	width of all icons
+ *		   icon_h -	height of all icons
+ *		   icon_d -	depth of all icons		   	    	
+ *                 resize - 	if TRUE the the list widget will
+ *                          	try to resize itself.
+ *	Returns: none.
+ *      NOTE:      If nitems of longest are <= 0 then they will be calculated.
+ *                 If nitems is <= 0 then the list needs to be NULL terminated.
+ */
+
+void XawIconListChange(w, iconlist, nitems, longest, icon_w, icon_h, icon_d, resize_it)
+Widget		w;
+XawIconList	*iconlist;
+int		nitems, longest, icon_w, icon_h, icon_d;
+Boolean		resize_it;
+{
+    ListWidget lw = (ListWidget) w;
+    int		   text_height;
+
+    /* Store the new values */
+
+    if (nitems < 0) nitems = 0;
+    if (longest < 0) longest = 0;
+
+    lw->list.iconList	 = iconlist;
+    lw->list.list	 = NULL;
+    lw->list.nitems	 = nitems;
+    lw->list.longest	 = longest;
+    lw->list.icon_width  = icon_w;
+    lw->list.icon_height = icon_h;
+    lw->list.icon_depth  = icon_d;
+
+    /* If the user passes 0 meaning "calculate it", it must be free */
+
+    if (longest != 0) lw->list.freedoms |= LongestLock;
+    else lw->list.freedoms &= ~LongestLock;
+
+    /* else - still resize if its not locked */
+
+    if (resize_it) lw->list.freedoms &= ~WidthLock & ~HeightLock;
+
+    /* Set row height, if required */
+
+    text_height = lw->list.font->max_bounds.ascent + lw->list.font->max_bounds.descent;
+    if (lw->list.icon_height > text_height)
+      lw->list.row_height = lw->list.icon_height + lw->list.row_space;
+
+    /* Redisplay the list */
+
+    ResetList(w, WidthFree(w), HeightFree(w));
+    lw->list.is_highlighted = lw->list.highlight = NO_HIGHLIGHT;
+    if (XtIsRealized(w)) Redisplay(w, (XEvent *)NULL, (Region)NULL);
 }
 
 /*	Function Name: XawListUnhighlight
@@ -949,13 +1173,8 @@ Boolean resize_it;
  *	Returns: none.
  */
 
-void
-#if NeedFunctionPrototypes
-XawListUnhighlight(Widget w)
-#else
-XawListUnhighlight(w)
+void XawListUnhighlight(w)
 Widget w;
-#endif
 {
     ListWidget lw = ( ListWidget ) w;
 
@@ -971,14 +1190,9 @@ Widget w;
  *	Returns: none.
  */
 
-void
-#if NeedFunctionPrototypes
-XawListHighlight(Widget w, int item)
-#else
-XawListHighlight(w, item)
+void XawListHighlight(w, item)
 Widget w;
 int item;
-#endif
 {
     ListWidget lw = ( ListWidget ) w;
     
@@ -986,7 +1200,7 @@ int item;
         lw->list.highlight = item;
         if (lw->list.is_highlighted != NO_HIGHLIGHT)
             PaintItemName(w, lw->list.is_highlighted);  /* Unhighlight. */
-	PaintItemName(w, item); /* HIGHLIGHT this one. */ 
+	PaintItemName(w, item);				/* HIGHLIGHT this one. */ 
     }
 }
 
@@ -996,25 +1210,22 @@ int item;
  *	Returns: the info about the currently highlighted object.
  */
 
-XawListReturnStruct *
-#if NeedFunctionPrototypes
-XawListShowCurrent(Widget w)
-#else
-XawListShowCurrent(w)
+XawListReturnStruct *XawListShowCurrent(w)
 Widget w;
-#endif
 {
-    ListWidget lw = ( ListWidget ) w;
+    ListWidget lw = (ListWidget) w;
     XawListReturnStruct * ret_val;
 
-    ret_val = (XawListReturnStruct *) XtMalloc (sizeof (XawListReturnStruct));
-    
-    ret_val->list_index = lw->list.highlight;
-    if (ret_val->list_index == XAW_LIST_NONE)
-      ret_val->string = "";
-    else
-      ret_val->string = lw->list.list[ ret_val->list_index ];
+    ret_val = (XawListReturnStruct *)XtMalloc(sizeof (XawListReturnStruct));
 
+    /* With this change we let it crash outside instead of inside Xaw :o) */
+
+    if (ret_val) {
+      ret_val->list_index = lw->list.highlight;
+      if (ret_val->list_index == XAW_LIST_NONE) *ret_val->string = '\0';
+      else ret_val->string = lw->list.iconList ? lw->list.iconList[ret_val->list_index].string
+					       : lw->list.list[ret_val->list_index];
+    }
     return(ret_val);
 }
 
