@@ -1,30 +1,27 @@
 /*
- * $XConsortium: Panner.c,v 1.45 92/03/03 13:52:26 converse Exp $
+ * $TOG: Panner.c /main/54 1998/02/11 14:54:13 kaleb $
  *
- * Copyright 1989 Massachusetts Institute of Technology
- *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  M.I.T. makes no representations about the
- * suitability of this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
- *
- * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
- * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
- * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+Copyright 1989, 1994, 1998  The Open Group
+
+All Rights Reserved.
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of The Open Group shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from The Open Group.
  *
  * Author:  Jim Fulton, MIT X Consortium
  *
- * 3D extensions added : Roland Krause 1997
- * Adaption to the new Add3dExt concept: Roland Krause 1999
- *
+ * This file contains modifications for XawPlus, Roland Krause 2000
  */
 
 #include <X11/IntrinsicP.h>		/* for toolkit routines */
@@ -36,7 +33,9 @@
 #include <ctype.h>			/* for isascii() etc. */
 #include <stdlib.h>			/* for atof() */
 
-#if defined(ISC) && defined(SYSV) && defined(SYSV386) && __STDC__
+extern Bool XmuDistinguishablePixels(); /* not defined in any Xmu headers */
+
+#if defined(ISC) && __STDC__ && !defined(ISC30)
 extern double atof(char *);
 #endif
 
@@ -146,9 +145,6 @@ PannerClassRec pannerClassRec = {
     /* display_accelerator	*/	XtInheritDisplayAccelerator,
     /* extension		*/	NULL
   },
-  { /* add3dExt fields */
-    /* unused			*/	0
-  },
   { /* simple fields */
     /* change_sensitive		*/	XtInheritChangeSensitive
   },
@@ -167,7 +163,7 @@ WidgetClass pannerWidgetClass = (WidgetClass) &pannerClassRec;
  *****************************************************************************/
 
 static void check_knob (pw, knob)
-    register PannerWidget pw;
+    PannerWidget pw;
     Boolean knob;
 {
     Position pad = pw->panner.internal_border * 2;
@@ -291,7 +287,7 @@ static Boolean get_event_xy (pw, event, x, y)
 }
 
 static int parse_page_string (s, pagesize, canvassize, relative)
-    register char *s;
+    char *s;
     int pagesize, canvassize;
     Boolean *relative;
 {
