@@ -182,7 +182,7 @@ ScrollbarClassRec scrollbarClassRec = {
     /* num_resources    */	XtNumber(resources),
     /* xrm_class        */	NULLQUARK,
     /* compress_motion	*/	TRUE,
-    /* compress_exposure*/	TRUE,
+    /* compress_exposure*/	XtExposeCompressSeries,
     /* compress_enterleave*/	TRUE,
     /* visible_interest */	FALSE,
     /* destroy          */	Destroy,
@@ -200,9 +200,6 @@ ScrollbarClassRec scrollbarClassRec = {
     /* display_accelerator*/	XtInheritDisplayAccelerator,
     /* extension        */	NULL
   },
-  { /* add3dExt fields */
-    /* unused		*/	0
-  },
   { /* simple fields */
     /* change_sensitive	*/	XtInheritChangeSensitive
   },
@@ -239,7 +236,7 @@ static void FillArea (sbw, top, bottom, fill)
 
     if (bottom <= 0 || bottom <= top) 
 	return;
-    if ((sw = sbw->add3dExt.borderWidth) < 0) 
+    if ((sw = sbw->simple.borderWidth) < 0) 
 	sw = 0;
     margin = MARGIN (sbw);
     floor = sbw->scrollbar.length - margin;
@@ -258,7 +255,7 @@ static void FillArea (sbw, top, bottom, fill)
     if (lh <= 0 || lw <= 0) return;
     if (fill) {
 	XFillRectangle(XtDisplay((Widget) sbw), XtWindow((Widget) sbw), 
-			sbw->add3dExt.backgrGC,
+			sbw->simple.backgrGC,
 			lx, ly, (unsigned int) lw, (unsigned int) lh);
     } else {
 	XClearArea (XtDisplay((Widget) sbw), XtWindow((Widget) sbw), 
@@ -275,7 +272,7 @@ static void PaintThumb (sbw, event)
     ScrollbarWidget sbw;
     XEvent *event; 
 {
-    Dimension s                   = sbw->add3dExt.borderWidth;
+    Dimension s                   = sbw->simple.borderWidth;
     Position  oldtop              = sbw->scrollbar.topLoc;
     Position  oldbot              = oldtop + sbw->scrollbar.shownLength;
     Dimension margin              = MARGIN (sbw);
@@ -286,8 +283,8 @@ static void PaintThumb (sbw, event)
     newtop = margin + (int)(tzl * sbw->scrollbar.top);
     newbot = newtop + (int)(tzl * sbw->scrollbar.shown);
     if (sbw->scrollbar.shown < 1.) newbot++;
-    if (newbot < newtop + (int)sbw->scrollbar.min_thumb + 2 * (int)sbw->add3dExt.borderWidth) 
-      newbot = newtop + sbw->scrollbar.min_thumb + 2 * sbw->add3dExt.borderWidth;
+    if (newbot < newtop + (int)sbw->scrollbar.min_thumb + 2 * (int)sbw->simple.borderWidth) 
+      newbot = newtop + sbw->scrollbar.min_thumb + 2 * sbw->simple.borderWidth;
     if ( newbot >= floor ) {
 	newtop = floor-(newbot-newtop)+1;
 	newbot = floor;
@@ -313,7 +310,7 @@ static void PaintArrows(sbw)
 ScrollbarWidget sbw;
 {
     XPoint    pt[20];
-    Dimension s   = sbw->add3dExt.borderWidth;
+    Dimension s   = sbw->simple.borderWidth;
     Dimension t   = sbw->scrollbar.thickness;
     Dimension l   = sbw->scrollbar.length;
     Dimension tms = t - s, lms = l - s;
@@ -371,23 +368,23 @@ ScrollbarWidget sbw;
 	 * swap the highlight and shadow GCs.
 	 */
 	if (sbw->scrollbar.scroll_mode != 1) {
-	  XFillPolygon(dpy, win, sbw->add3dExt.highlightGC, pt, 4, Complex, CoordModeOrigin);
-	  XFillPolygon(dpy, win, sbw->add3dExt.shadowGC, pt + 4, 6, Complex, CoordModeOrigin);
+	  XFillPolygon(dpy, win, sbw->simple.highlightGC, pt, 4, Complex, CoordModeOrigin);
+	  XFillPolygon(dpy, win, sbw->simple.shadowGC, pt + 4, 6, Complex, CoordModeOrigin);
 	}
 	else {
-	  XFillPolygon(dpy, win, sbw->add3dExt.shadowGC, pt, 4, Complex, CoordModeOrigin);
-	  XFillPolygon(dpy, win, sbw->add3dExt.highlightGC, pt + 4, 6, Complex, CoordModeOrigin);
+	  XFillPolygon(dpy, win, sbw->simple.shadowGC, pt, 4, Complex, CoordModeOrigin);
+	  XFillPolygon(dpy, win, sbw->simple.highlightGC, pt + 4, 6, Complex, CoordModeOrigin);
 	}
 	/* Draw the lower or right arrow. If the arrow is pressed,
 	 * swap the highlight and shadow GCs.
 	 */
 	if (sbw->scrollbar.scroll_mode != 3) {
-	  XFillPolygon(dpy, win, sbw->add3dExt.highlightGC, pt + 10, 6, Complex, CoordModeOrigin);
-	  XFillPolygon(dpy, win, sbw->add3dExt.shadowGC, pt + 16, 4, Complex, CoordModeOrigin);
+	  XFillPolygon(dpy, win, sbw->simple.highlightGC, pt + 10, 6, Complex, CoordModeOrigin);
+	  XFillPolygon(dpy, win, sbw->simple.shadowGC, pt + 16, 4, Complex, CoordModeOrigin);
 	}
 	else {
-	  XFillPolygon(dpy, win, sbw->add3dExt.shadowGC, pt + 10, 6, Complex, CoordModeOrigin);
-	  XFillPolygon(dpy, win, sbw->add3dExt.highlightGC, pt + 16, 4, Complex, CoordModeOrigin);
+	  XFillPolygon(dpy, win, sbw->simple.shadowGC, pt + 10, 6, Complex, CoordModeOrigin);
+	  XFillPolygon(dpy, win, sbw->simple.highlightGC, pt + 16, 4, Complex, CoordModeOrigin);
 	}
     }
 }
@@ -549,17 +546,17 @@ static void Redisplay (w, event, region)
       XawSunkenRectangle(w, 0, 0, w->core.width, w->core.height);
       if (sbw->scrollbar.orientation == XtorientHorizontal) {
 	x = sbw->scrollbar.topLoc;
-	y = sbw->add3dExt.borderWidth;
+	y = sbw->simple.borderWidth;
 	width = sbw->scrollbar.shownLength;
-	height = sbw->core.height - 2 * sbw->add3dExt.borderWidth;
+	height = sbw->core.height - 2 * sbw->simple.borderWidth;
       } else {
-	x = sbw->add3dExt.borderWidth;
+	x = sbw->simple.borderWidth;
 	y = sbw->scrollbar.topLoc;
-	width = sbw->core.width - 2 * sbw->add3dExt.borderWidth;
+	width = sbw->core.width - 2 * sbw->simple.borderWidth;
 	height = sbw->scrollbar.shownLength;
       }
       if (region == NULL || XRectInRegion (region, x, y, width, height) != RectangleOut) {
-	sbw->scrollbar.topLoc = sbw->add3dExt.borderWidth;
+	sbw->scrollbar.topLoc = sbw->simple.borderWidth;
 	PaintThumb (sbw, event); 
       }
       /* we'd like to be region aware here!!!! */
@@ -889,22 +886,6 @@ static void NotifyThumb (w, event, params, num_params )
        compatibility on those architectures for which it work{s,ed};
        the intent is to pass a (truncated) float by value. */
 
-    /* This corrects for rounding errors: If the thumb is moved to the end of
-       the scrollable area sometimes the last line/column is not displayed.
-       This can happen when the integer number of the top line or leftmost
-       column to be be displayed is calculated from the float value
-       sbw->scrollbar.top. The numerical error of this rounding problem is
-       very small. We therefore add a small value which then forces the
-       next line/column (the correct one) to be used. Since we can expect
-       that the resolution of display screens will not be higher then
-       10000 text lines/columns we add 1/10000 to the top position. The
-       intermediate variable `top' is used to avoid erroneous summing up
-       corrections (can this happen at all?). If the arrows are not displayed
-       there is no problem since in this case there is always a constant
-       integer number of pixels the thumb must be moved in order to scroll
-       to the next line/column. */
-
-    top += 0.0001;
     XtCallCallbacks (w, XtNthumbProc, *(XtPointer*)&top);
     XtCallCallbacks (w, XtNjumpProc, (XtPointer)&top);
 }
@@ -929,7 +910,7 @@ void XawScrollbarSetThumb (w, top, shown)
     fprintf(stderr,"< XawScrollbarSetThumb w=%p, top=%f, shown=%f\n", w,top,shown);
 #endif
 
-    if (sbw->scrollbar.scroll_mode == (char)2) return;      /* if still thumbing */
+    if (sbw->scrollbar.scroll_mode == (char)2) return;     /* if still thumbing */
     sbw->scrollbar.top = (top > 1.0) ? 1.0 :
 				(top >= 0.0) ? top : sbw->scrollbar.top;
     sbw->scrollbar.shown = (shown > 1.0) ? 1.0 :
