@@ -1,24 +1,23 @@
-/* $XConsortium: SmeBSB.c,v 1.16 91/03/15 15:59:41 gildea Exp $ */
+/* $TOG: SmeBSB.c /main/23 1998/06/24 11:53:39 kaleb $ */
 
 /*
- * Copyright 1989 Massachusetts Institute of Technology
- *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  M.I.T. makes no representations about the
- * suitability of this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
- *
- * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
- * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
- * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+Copyright 1989, 1994, 1998  The Open Group
+
+All Rights Reserved.
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of The Open Group shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from The Open Group.
  */
 
 /*
@@ -30,15 +29,20 @@
  *          MIT X Consortium 
  *          kit@expo.lcs.mit.edu
  *
- * 3D extensions added : Roland Krause 1997
- * XPM extensions added: Roland Krause 1997
- * Installation of XPM type converters added: Roland Krause 1999
+ * This file contains modifications for XawPlus, Roland Krause 2000
  */
 
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Xos.h>
 
+/* needed for abs() */
+#ifdef X_NOT_STDC_ENV
+extern int abs();
+#else
+#include <stdlib.h>
+#endif
+
 #include <X11/Xmu/Drawing.h>
 
 #include <X11/XawPlus/XawInit.h>
@@ -76,6 +80,8 @@ static XtResource resources[] = {
      offset(foreground), XtRString, XtDefaultForeground},
   {XtNfont,  XtCFont, XtRFontStruct, sizeof(XFontStruct *),
      offset(font), XtRString, XtDefaultFont},
+  {XtNfontSet,  XtCFontSet, XtRFontSet, sizeof(XFontSet ),
+     offset(fontset),XtRString, XtDefaultFontSet},
 };   
 #undef offset
 
@@ -131,15 +137,13 @@ SmeBSBClassRec smeBSBClassRec = {
     /* display_accelerator*/    NULL,
     /* extension	  */    NULL
   },{
-    /* Menu Entry Fields */
-      
-    /* highlight */             FlipColors,
-    /* unhighlight */           FlipColors,
-    /* notify */		XtInheritNotify,		
-    /* extension	  */    NULL
+    /* SimpleMenuClass Fields */
+    /* highlight          */	FlipColors,
+    /* unhighlight        */	FlipColors,
+    /* notify             */	XtInheritNotify,		
+    /* extension	  */	NULL
   }, {
-    /* BSB Menu entry Fields */  
-
+    /* BSBClass Fields */  
     /* extension	  */    NULL
   }
 };
@@ -239,10 +243,18 @@ Region region;
     int		 font_ascent, font_descent, x_loc, y_loc;
     Boolean	 insensitive;
 
-    entry->sme_bsb.set_values_area_cleared = FALSE;    
-    font_ascent = entry->sme_bsb.font->max_bounds.ascent;
-    font_descent = entry->sme_bsb.font->max_bounds.descent;
+    int	fontset_ascent, fontset_descent;
+    XFontSetExtents *ext = XExtentsOfFontSet(entry->sme_bsb.fontset);
 
+    entry->sme_bsb.set_values_area_cleared = FALSE;    
+    if ( entry->sme.international == True ) {
+        fontset_ascent = abs(ext->max_ink_extent.y);
+        fontset_descent = ext->max_ink_extent.height - fontset_ascent;
+    }
+    else { /*else, compute size from font like R5*/
+        font_ascent = entry->sme_bsb.font->max_bounds.ascent;
+        font_descent = entry->sme_bsb.font->max_bounds.descent;
+    }
     y_loc = entry->rectangle.y;
     x_loc = entry->rectangle.x;
     
@@ -263,43 +275,82 @@ Region region;
     }
     
     if (entry->sme_bsb.label != NULL) {
-	int x_loc = entry->sme_bsb.left_margin;
-	int len = strlen(entry->sme_bsb.label);
-	char * label = entry->sme_bsb.label;
+	int  len    = strlen(entry->sme_bsb.label);
+	char *label = entry->sme_bsb.label;
+	x_loc	    = entry->sme_bsb.left_margin;
+
 
 	switch(entry->sme_bsb.justify) {
 	    int width, t_width;
 
 	case XtJustifyCenter:
-	    t_width = XTextWidth(entry->sme_bsb.font, label, len);
-	    width = entry->rectangle.width - (entry->sme_bsb.left_margin +
+            if ( entry->sme.international == True ) {
+	        t_width = XmbTextEscapement(entry->sme_bsb.fontset,label,len);
+                width = entry->rectangle.width - (entry->sme_bsb.left_margin +
 					      entry->sme_bsb.right_margin);
+            }
+            else {
+	        t_width = XTextWidth(entry->sme_bsb.font, label, len);
+	        width = entry->rectangle.width - (entry->sme_bsb.left_margin +
+					      entry->sme_bsb.right_margin);
+            }
 	    x_loc += (width - t_width)/2;
 	    break;
-
 	case XtJustifyRight:
-	    t_width = XTextWidth(entry->sme_bsb.font, label, len);
-	    x_loc = entry->rectangle.width - (entry->sme_bsb.right_margin + t_width);
+            if ( entry->sme.international == True ) {
+                t_width = XmbTextEscapement(entry->sme_bsb.fontset,label,len);
+                x_loc = entry->rectangle.width - ( entry->sme_bsb.right_margin
+						 + t_width );
+            }
+            else {
+	        t_width = XTextWidth(entry->sme_bsb.font, label, len);
+	        x_loc = entry->rectangle.width - ( entry->sme_bsb.right_margin
+						 + t_width );
+            }
 	    break;
-
 	case XtJustifyLeft:
 	default:
 	    break;
 	}
+	/* this will center the text in the gadget top-to-bottom */
 
-	y_loc += ((int)entry->rectangle.height - 
-		  (font_ascent + font_descent)) / 2 + font_ascent;
-
-	if (insensitive == TRUE) {
-	  XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.lgray_gc,
-		      x_loc+1, y_loc+1, label, len);
-	  XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.gray_gc,
-		      x_loc, y_loc, label, len);
-	}
-	else XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), gc,
-			 x_loc, y_loc, label, len);
+        if ( entry->sme.international==True )
+	{
+          y_loc += ((int)entry->rectangle.height - 
+		    (fontset_ascent + fontset_descent)) / 2 + fontset_ascent;
+	  if (insensitive == TRUE)
+	  {
+	    /* Draw the multi byte string in the insensitive style
+	     */
+	    XmbDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.fontset,
+			  entry->sme_bsb.lgray_gc, x_loc+1, y_loc+1, label, len);
+	    XmbDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.fontset,
+			  entry->sme_bsb.gray_gc, x_loc, y_loc, label, len);
+	  }
+	  /* Draw the multi byte string in normal or revert style
+	   */
+          else XmbDrawString(XtDisplayOfObject(w), XtWindowOfObject(w),
+			     entry->sme_bsb.fontset, gc, x_loc, y_loc, label, len);
+        }
+        else
+	{
+          y_loc += ((int)entry->rectangle.height - 
+		    (font_ascent + font_descent)) / 2 + font_ascent;
+	  if (insensitive == TRUE)
+	  {
+	    /* Draw the string in the insensitive style
+	     */
+	    XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.lgray_gc,
+		        x_loc+1, y_loc+1, label, len);
+	    XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), entry->sme_bsb.gray_gc,
+		        x_loc, y_loc, label, len);
+	  }
+	  /* Draw the string in normal or revert style
+	   */
+          else XDrawString(XtDisplayOfObject(w), XtWindowOfObject(w), gc,
+		    	   x_loc, y_loc, label, len);
+        }
     }
-
     DrawBitmaps(w, gc);
 }
 
@@ -346,7 +397,7 @@ Cardinal *num_args;
 	ret_val = TRUE;
     }
 
-    if ((old_entry->sme_bsb.font != entry->sme_bsb.font) ||
+    if (((old_entry->sme_bsb.font != entry->sme_bsb.font) && (old_entry->sme.international == False)) ||
 	(old_entry->sme_bsb.foreground != entry->sme_bsb.foreground) ||
 	(old_entry->sme.highlightColor != entry->sme.highlightColor) ||
 	(old_entry->sme.shadowColor != entry->sme.shadowColor)) {
@@ -355,6 +406,11 @@ Cardinal *num_args;
 	ret_val = TRUE;
     }
 
+    if ( ( old_entry->sme_bsb.fontset != entry->sme_bsb.fontset) &&
+				(old_entry->sme.international == True ) )
+        /* DONT changes the GCs, because the fontset is not in them. */
+        ret_val = TRUE;
+
     if (ret_val) {
 	GetDefaultSize(new, &(entry->rectangle.width), &(entry->rectangle.height));
 	entry->sme_bsb.set_values_area_cleared = TRUE;
@@ -449,19 +505,33 @@ Dimension * width, * height;
 {
     SmeBSBObject entry = (SmeBSBObject) w;
 
-    if (entry->sme_bsb.label == NULL) 
-	*width = 0;
-    else
-	*width = XTextWidth(entry->sme_bsb.font, entry->sme_bsb.label,
+    if ( entry->sme.international == True ) {
+        XFontSetExtents *ext = XExtentsOfFontSet(entry->sme_bsb.fontset);
+        if (entry->sme_bsb.label == NULL) 
+	    *width = 0;
+        else
+	    *width = XmbTextEscapement(entry->sme_bsb.fontset, entry->sme_bsb.label,
+			    strlen(entry->sme_bsb.label));
+        *width += entry->sme_bsb.left_margin + entry->sme_bsb.right_margin;
+        *height = ext->max_ink_extent.height;
+        *height = ((int)*height * ( ONE_HUNDRED + 
+			        entry->sme_bsb.vert_space )) / ONE_HUNDRED;
+    }
+    else {
+        if (entry->sme_bsb.label == NULL) 
+	    *width = 0;
+        else
+	    *width = XTextWidth(entry->sme_bsb.font, entry->sme_bsb.label,
 			    strlen(entry->sme_bsb.label));
 
-    *width += entry->sme_bsb.left_margin + entry->sme_bsb.right_margin;
+        *width += entry->sme_bsb.left_margin + entry->sme_bsb.right_margin;
     
-    *height = (entry->sme_bsb.font->max_bounds.ascent +
+        *height = (entry->sme_bsb.font->max_bounds.ascent +
 	       entry->sme_bsb.font->max_bounds.descent);
 
-    *height = ((int)*height * ( ONE_HUNDRED + 
+        *height = ((int)*height * ( ONE_HUNDRED + 
 			        entry->sme_bsb.vert_space )) / ONE_HUNDRED;
+    }
 }
 
 /*      Function Name: DrawBitmaps
@@ -480,9 +550,11 @@ GC gc;
     
     if ( (entry->sme_bsb.left_bitmap == None) && 
 	 (entry->sme_bsb.right_bitmap == None) ) return;
+
 /*
  * Draw Left Bitmap or Pixmap
  */
+
   if (entry->sme_bsb.left_bitmap != None) {
     x_loc = (int)(entry->sme_bsb.left_margin - entry->sme_bsb.left_bitmap_width) / 2;
     y_loc = entry->rectangle.y + (int)(entry->rectangle.height -
@@ -505,9 +577,12 @@ GC gc;
       XSetClipMask(XtDisplayOfObject(w), gc, None);
     }
   }
+
 /*
  * Draw Right Bitmap or Pixmap
  */
+
+
   if (entry->sme_bsb.right_bitmap != None) {
     x_loc = entry->rectangle.width - (int)(entry->sme_bsb.right_margin +
 					   entry->sme_bsb.right_bitmap_width) / 2;
@@ -589,32 +664,60 @@ Widget w;
 {
     SmeBSBObject entry = (SmeBSBObject) w;    
     XGCValues values;
-    XtGCMask mask;
+    XtGCMask  mask;
 
-    /* Get GC with reverse colors */
+    if (entry->sme.international == True)
+    {
+      mask = GCForeground | GCBackground | GCGraphicsExposures;
+
+      /* Get GC with reverse colors */
     
-    values.foreground = XtParent(w)->core.background_pixel;
-    values.background = entry->sme_bsb.foreground;
-    values.font = entry->sme_bsb.font->fid;
-    values.graphics_exposures = FALSE;
-    mask        = GCForeground | GCBackground | GCFont | GCGraphicsExposures;
-    entry->sme_bsb.rev_gc = XtGetGC(w, mask, &values);
+      values.foreground = XtParent(w)->core.background_pixel;
+      values.background = entry->sme_bsb.foreground;
+      values.graphics_exposures = FALSE;
+      entry->sme_bsb.rev_gc = XtAllocateGC(w, 0, mask, &values, GCFont, 0);
 
-    /* Get GC with normal colors */
+      /* Get GC with normal colors */
 
-    values.foreground = entry->sme_bsb.foreground;
-    values.background = XtParent(w)->core.background_pixel;
-    entry->sme_bsb.norm_gc = XtGetGC(w, mask, &values);
+      values.foreground = entry->sme_bsb.foreground;
+      values.background = XtParent(w)->core.background_pixel;
+      entry->sme_bsb.norm_gc = XtAllocateGC(w, 0, mask, &values, GCFont, 0);
 
-    /* Get GCs to display insensitive entries */
+      /* Get GCs to display insensitive entries */
 
-    values.foreground = entry->sme.shadowColor;
-    entry->sme_bsb.gray_gc = XtGetGC(w, mask, &values);
+      values.foreground = entry->sme.shadowColor;
+      entry->sme_bsb.gray_gc = XtAllocateGC(w, 0, mask, &values, GCFont, 0);
 
-    values.foreground = entry->sme.highlightColor;
-    entry->sme_bsb.lgray_gc = XtGetGC(w, mask, &values);
+      values.foreground = entry->sme.highlightColor;
+      entry->sme_bsb.lgray_gc = XtAllocateGC(w, 0, mask, &values, GCFont, 0);
+    }
+    else
+    {
+      mask = GCForeground | GCBackground | GCGraphicsExposures | GCFont;
 
-    /* Get GC for flipping colors */
+      /* Get GC with reverse colors */
+
+      values.foreground = XtParent(w)->core.background_pixel;
+      values.background = entry->sme_bsb.foreground;
+      values.font = entry->sme_bsb.font->fid;
+      values.graphics_exposures = FALSE;
+      entry->sme_bsb.rev_gc = XtGetGC(w, mask, &values);
+
+      /* Get GC with normal colors */
+
+      values.foreground = entry->sme_bsb.foreground;
+      values.background = XtParent(w)->core.background_pixel;
+      entry->sme_bsb.norm_gc = XtGetGC(w, mask, &values);
+
+      /* Get GCs to display insensitive entries */
+
+      values.foreground = entry->sme.shadowColor;
+      entry->sme_bsb.gray_gc = XtGetGC(w, mask, &values);
+
+      values.foreground = entry->sme.highlightColor;
+      entry->sme_bsb.lgray_gc = XtGetGC(w, mask, &values);
+    }
+    /* Get GC for flipping colors, this GC is font independent ! */
 
     values.foreground = entry->sme_bsb.foreground ^ values.background;
     values.background = 0;
