@@ -1,27 +1,30 @@
-/* $XConsortium: TextSrc.c,v 1.11 91/02/20 17:58:08 converse Exp $ */
+/* $TOG: TextSrc.c /main/21 1998/05/25 08:17:30 kaleb $ */
+/*
+
+Copyright 1989, 1994, 1998  The Open Group
+
+All Rights Reserved.
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of The Open Group shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from The Open Group.
+
+This file contains modifications for XawPlus, Roland Krause 2000
+
+*/
 
 /*
- * Copyright 1989 Massachusetts Institute of Technology
- *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  M.I.T. makes no representations about the
- * suitability of this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
- *
- * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
- * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
- * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
  * Author:  Chris Peterson, MIT X Consortium.
- *
  * Much code taken from X11R3 String and Disk Sources.
  */
 
@@ -29,14 +32,16 @@
  * TextSrc.c - TextSrc object. (For use with the text widget).
  *
  */
-
 #include <stdio.h>
 #include <ctype.h>
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
+#include <X11/Xutil.h>
 #include <X11/XawPlus/XawInit.h>
 #include <X11/XawPlus/TextSrcP.h>
+#include <X11/Xmu/Atoms.h>
 #include <X11/Xmu/CharSet.h>
+#include "XawI18n.h"
 
 /****************************************************************
  *
@@ -120,7 +125,7 @@ static void
 ClassPartInitialize(wc)
 WidgetClass wc;
 {
-  register TextSrcObjectClass t_src, superC;
+  TextSrcObjectClass t_src, superC;
 
   t_src = (TextSrcObjectClass) wc;
   superC = (TextSrcObjectClass) t_src->object_class.superclass;
@@ -129,7 +134,6 @@ WidgetClass wc;
  * We don't need to check for null super since we'll get to TextSrc
  * eventually.
  */
-
     if (t_src->textSrc_class.Read == XtInheritRead) 
       t_src->textSrc_class.Read = superC->textSrc_class.Read;
 
@@ -173,7 +177,8 @@ int length;
 {
   XtAppError(XtWidgetToApplicationContext(w), 
 	     "TextSrc Object: No read function is defined.");
-  return(0L);
+
+  return( (XawTextPosition) 0 ); /* for gcc -Wall and lint */
 }
 
 /*	Function Name: Replace.
@@ -216,7 +221,8 @@ Boolean	              include;
 {
   XtAppError(XtWidgetToApplicationContext(w), 
 	     "TextSrc Object: No SCAN function is defined.");
-  return(0L);
+
+  return( (XawTextPosition) 0 ); /* for gcc -Wall and lint */
 }
 
 /*	Function Name: Search
@@ -276,10 +282,6 @@ Atom selection;
 }
 
 
-#define done(address, type) \
-        { toVal->size = sizeof(type); toVal->addr = (caddr_t) address; }
-
-
 static void CvtStringToEditMode(args, num_args, fromVal, toVal)
 XrmValuePtr args;		/* unused */
 Cardinal	*num_args;	/* unused */
@@ -289,7 +291,7 @@ XrmValuePtr	toVal;
   static XawTextEditType editType;
   static  XrmQuark  QRead, QAppend, QEdit;
   XrmQuark    q;
-  char        lowerName[BUFSIZ];
+  char        lowerName[40];
   static Boolean inited = FALSE;
     
   if ( !inited ) {
@@ -299,20 +301,28 @@ XrmValuePtr	toVal;
     inited = TRUE;
   }
 
-  XmuCopyISOLatin1Lowered (lowerName, (char *)fromVal->addr);
-  q = XrmStringToQuark(lowerName);
+  if (strlen ((char*) fromVal->addr) < sizeof lowerName) {
+    XmuCopyISOLatin1Lowered (lowerName, (char *)fromVal->addr);
+    q = XrmStringToQuark(lowerName);
 
-  if       (q == QRead)          editType = XawtextRead;
-  else if (q == QAppend)         editType = XawtextAppend;
-  else if (q == QEdit)           editType = XawtextEdit;
-  else {
-    done(NULL, 0);
+    if      (q == QRead)   editType = XawtextRead;
+    else if (q == QAppend) editType = XawtextAppend;
+    else if (q == QEdit)   editType = XawtextEdit;
+    else {
+      toVal->size = 0;
+      toVal->addr = NULL;
+      return;
+    }
+    toVal->size = sizeof editType;
+    toVal->addr = (XPointer) &editType;
     return;
   }
-  done(&editType, XawTextEditType);
-  return;
+  toVal->size = 0;
+  toVal->addr = NULL;
 }
 
+
+
 /************************************************************
  *
  * Public Functions.
@@ -342,6 +352,11 @@ int length;
 {
   TextSrcObjectClass class = (TextSrcObjectClass) w->core.widget_class;
 
+  if ( !XtIsSubclass( w, textSrcObjectClass ) )
+      XtErrorMsg("bad argument", "textSource", "XawError",
+		"XawTextSourceRead's 1st parameter must be subclass of asciiSrc.",
+		   NULL, NULL);
+
   return((*class->textSrc_class.Read)(w, pos, text, length));
 }
 
@@ -367,6 +382,11 @@ XawTextBlock *text;
 {
   TextSrcObjectClass class = (TextSrcObjectClass) w->core.widget_class;
 
+  if ( !XtIsSubclass( w, textSrcObjectClass ) )
+      XtErrorMsg("bad argument", "textSource", "XawError",
+		"XawTextSourceReplace's 1st parameter must be subclass of asciiSrc.",
+		   NULL, NULL);
+
   return((*class->textSrc_class.Replace)(w, startPos, endPos, text));
 }
 
@@ -410,6 +430,11 @@ Boolean	              include;
 {
   TextSrcObjectClass class = (TextSrcObjectClass) w->core.widget_class;
 
+  if ( !XtIsSubclass( w, textSrcObjectClass ) )
+      XtErrorMsg("bad argument", "textSource", "XawError",
+		"XawTextSourceScan's 1st parameter must be subclass of asciiSrc.",
+		   NULL, NULL);
+
   return((*class->textSrc_class.Scan)(w, position, type, dir, count, include));
 }
 
@@ -442,6 +467,11 @@ XawTextBlock *        text;
 {
   TextSrcObjectClass class = (TextSrcObjectClass) w->core.widget_class;
 
+  if ( !XtIsSubclass( w, textSrcObjectClass ) )
+      XtErrorMsg("bad argument", "textSource", "XawError",
+		"XawTextSourceSearch's 1st parameter must be subclass of asciiSrc.",
+		   NULL, NULL);
+
   return((*class->textSrc_class.Search)(w, position, dir, text));
 }
 
@@ -474,6 +504,11 @@ int * format;
 {
   TextSrcObjectClass class = (TextSrcObjectClass) w->core.widget_class;
 
+  if ( !XtIsSubclass( w, textSrcObjectClass ) )
+      XtErrorMsg("bad argument", "textSource", "XawError",
+		"XawTextSourceConvertSelectionXawTextSourceConvertSelection's 1st parameter must be subclass of asciiSrc.",
+		   NULL, NULL);
+
   return((*class->textSrc_class.ConvertSelection)(w, selection, target, type,
 						  value, length, format));
 }
@@ -499,7 +534,126 @@ Atom selection;
 {
   TextSrcObjectClass class = (TextSrcObjectClass) w->core.widget_class;
 
+  if ( !XtIsSubclass( w, textSrcObjectClass ) )
+      XtErrorMsg("bad argument", "textSource", "XawError",
+		"'s 1st parameter must be subclass of asciiSrc.",
+		   NULL, NULL);
+
   (*class->textSrc_class.SetSelection)(w, left, right, selection);
 }
 
+/********************************************************************
+ *
+ *      External Functions for Multi Text.
+ *
+ ********************************************************************/
+
+/*
+ * TextFormat(): 
+ *   returns the format of text: FMT8BIT or FMTWIDE. 
+ *  
+ */
+XrmQuark
+#if NeedFunctionPrototypes
+_XawTextFormat(TextWidget tw)
+#else
+_XawTextFormat(tw)
+    TextWidget tw;
+#endif
+{
+  return (((TextSrcObject)(tw->text.source))->textSrc.text_format);
+}
+
+
+/* _XawTextWCToMB():
+ *   convert the wchar string to external encoding.
+ *   The caller is responsible for freeing both the source and ret string.
+ *
+ * wstr       - source wchar string.
+ * len_in_out - lengh of string.
+ *              As In, length of source wchar string, measured in wchar.
+ *              As Out, length of returned string. 
+ */
+
+
+char *
+_XawTextWCToMB( d, wstr, len_in_out )
+    Display*	d;
+    wchar_t*	wstr;
+    int*	len_in_out;
+
+{
+    XTextProperty textprop;
+    if (XwcTextListToTextProperty(d, (wchar_t**)&wstr, 1,
+      XTextStyle, &textprop) < Success) {
+      XtWarningMsg("convertError", "textSource", "XawError",
+                 "Non-character code(s) in buffer.", NULL, NULL);
+      *len_in_out = 0;
+      return( NULL );
+    }
+    *len_in_out = textprop.nitems;
+    return((char *)textprop.value);
+}
+
+
+/* _XawTextMBToWC():
+ *   convert the string to internal processing codeset WC.
+ *   The caller is responsible for freeing both the source and ret string.
+ * 
+ * str        - source string.
+ * len_in_out - lengh of string.
+ *              As In, it is length of source string.
+ *              As Out, it is length of returned string, measured in wchar.
+ */
+
+wchar_t* _XawTextMBToWC( d, str, len_in_out )
+Display		*d;
+char		*str;
+int		*len_in_out;
+{
+  if (*len_in_out != 0) {
+    XTextProperty textprop;
+    char *buf;
+    wchar_t **wlist, *wstr;
+    int count;
+    buf = XtMalloc(*len_in_out + 1);
+    if (!buf) {
+	XtErrorMsg("convertError", "multiSourceCreate", "XawError",
+		 "No Memory", NULL, NULL);
+	*len_in_out = 0;
+	return (NULL);   /* The above function doesn't really return. */
+    }
+    strncpy(buf, str, *len_in_out);
+    *(buf + *len_in_out) = '\0';
+    if (XmbTextListToTextProperty(d, &buf, 1, XTextStyle, &textprop)
+			!= Success) {
+	XtWarningMsg("convertError", "textSource", "XawError",
+		 "No Memory, or Locale not supported.", NULL, NULL);
+	XtFree(buf);
+	*len_in_out = 0;
+	return (NULL);
+    }
+    XtFree(buf);
+    if (XwcTextPropertyToTextList(d, &textprop,
+			(wchar_t***)&wlist, &count) != Success) {
+	XtWarningMsg("convertError", "multiSourceCreate", "XawError",
+		 "Non-character code(s) in source.", NULL, NULL);
+	*len_in_out = 0;
+	return (NULL);
+    }
+    wstr = (wchar_t *) XtMalloc((wcslen(wlist[0]) + 1) * sizeof(wchar_t));
+    if (wstr == (wchar_t *) NULL) {
+	XwcFreeStringList(wlist);
+	XtErrorMsg("convertError", "multiSourceCreate", "XawError",
+		   "No Memory", NULL, NULL);
+	*len_in_out = 0;
+	return (NULL);   /* The above function doesn't really return. */
+    }
+    wcscpy(wstr, wlist[0]);
+    *len_in_out = wcslen(wstr);
+    XwcFreeStringList(wlist);
+    return(wstr); 	/* This return is missed in original Xaw sources */
+  }
+  return(NULL);
+}
 
