@@ -1,28 +1,28 @@
-/* $XConsortium: Toggle.c,v 1.24 91/07/25 14:07:48 converse Exp $ */
+/* $TOG: Toggle.c /main/30 1998/02/11 14:55:52 kaleb $ */
 
 /*
- * Copyright 1989 Massachusetts Institute of Technology
- *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  M.I.T. makes no representations about the
- * suitability of this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
- *
- * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
- * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
- * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * XawPlus extensions added : Roland Krause 1996
- * Changes for the new highlight mechanism in Command: Roland Krause 1999
- */
+
+Copyright 1989, 1994, 1998  The Open Group
+
+All Rights Reserved.
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of The Open Group shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from The Open Group.
+
+This file contains modifications for XawPlus, Roland Krause 2000
+
+*/
 
 /*
  * Toggle.c - Toggle button widget
@@ -130,9 +130,6 @@ ToggleClassRec toggleClassRec = {
     XtInheritDisplayAccelerator,	/* display_accelerator	  */
     NULL				/* extension		  */
   },  /* CoreClass fields initialization */
-  {
-    0					/* unused		  */
-  },  /* add3dExt fields initialization */
   {
     XtInheritChangeSensitive		/* change_sensitive	  */ 
   },  /* SimpleClass fields initialization */
@@ -173,7 +170,7 @@ ClassInit()
   XawInitializeWidgetSet();
   XtSetTypeConverter(XtRString, XtRWidget, XmuNewCvtStringToWidget,
 		     parentCvtArgs, XtNumber(parentCvtArgs), XtCacheNone,
-		     NULL);
+		     (XtDestructor)NULL);
 /* 
  * Find the set and unset actions in the command widget's action table. 
  */
@@ -199,8 +196,8 @@ ClassInit()
 
 static void Initialize(request, new, args, num_args)
  Widget request, new;
-ArgList args;
-Cardinal *num_args;
+ ArgList args;
+ Cardinal *num_args;
 {
     ToggleWidget tw = (ToggleWidget) new;
     ToggleWidget tw_req = (ToggleWidget) request;
@@ -216,7 +213,7 @@ Cardinal *num_args;
       else
 	AddToRadioGroup( GetRadioGroup(tw->toggle.widget), new);
     }      
-    XtAddCallback(new, XtNdestroyCallback, ToggleDestroy, NULL);
+    XtAddCallback(new, XtNdestroyCallback, ToggleDestroy, (XtPointer)NULL);
 
 /*
  * Command widget assumes that the widget is unset, so we only 
@@ -230,7 +227,7 @@ Cardinal *num_args;
  */
 
     if (tw_req->command.set)
-      ToggleSet(new, NULL, NULL, 0);
+      ToggleSet(new, (XEvent *)NULL, (String *)NULL, (Cardinal *)NULL);
 }
 
 /************************************************************
@@ -278,7 +275,9 @@ String *params;		/* unused */
 Cardinal *num_params;	/* unused */
 {
   ToggleWidget tw = (ToggleWidget) w;
-  XtCallCallbacks(w, XtNcallback, (XtPointer)(tw->command.set));
+  long antilint = tw->command.set;
+
+  XtCallCallbacks( w, XtNcallback, (XtPointer) antilint );
 }
 
 /************************************************************
@@ -289,8 +288,10 @@ Cardinal *num_params;	/* unused */
 
 /* ARGSUSED */
 static Boolean 
-SetValues (current, request, new)
+SetValues (current, request, new, args, num_args)
 Widget current, request, new;
+ArgList args;
+Cardinal *num_args;
 {
     ToggleWidget oldtw = (ToggleWidget) current;
     ToggleWidget tw = (ToggleWidget) new;
@@ -304,7 +305,7 @@ Widget current, request, new;
 
     if (oldtw->command.set != tw->command.set) {
 	tw->command.set = oldtw->command.set;
-	Toggle(new, NULL, NULL, 0); /* Does a redisplay. */
+	Toggle(new, (XEvent *)NULL, (String *)NULL, (Cardinal *)0);
     }
     return(FALSE);
 }
@@ -327,7 +328,7 @@ XtPointer junk, garbage;
 
 /************************************************************
  *
- * Below are all the private proceedures that handle 
+ * Below are all the private procedures that handle 
  * radio toggle buttons.
  *
  ************************************************************/
@@ -367,12 +368,12 @@ Widget w1, w2;
   ToggleWidget tw2 = (ToggleWidget) w2;
 
   if ( (tw1->toggle.radio_group != NULL) || (tw2->toggle.radio_group != NULL) ) {
-    sprintf(error_buf, "%s %s", "Toggle Widget Error - Attempting",
+    (void) sprintf(error_buf, "%s %s", "Toggle Widget Error - Attempting",
 	    "to create a new toggle group, when one already exists.");
     XtWarning(error_buf);
   }
 
-  AddToRadioGroup( NULL, w1 );
+  AddToRadioGroup( (RadioGroup *)NULL, w1 );
   AddToRadioGroup( GetRadioGroup(w1), w2 );
 }
 
@@ -431,7 +432,7 @@ Widget w;
     ToggleWidget local_tog = (ToggleWidget) group->widget;
     if ( local_tog->command.set ) {
       class->toggle_class.Reset(group->widget, NULL, NULL, 0);
-      Notify( group->widget, NULL, NULL, 0);
+      Notify(group->widget, (XEvent *)NULL, (String *)NULL, (Cardinal *)NULL);
     }
     group = group->next;
   }
@@ -546,14 +547,14 @@ XtPointer radio_data;
   RadioGroup * group;
   ToggleWidget local_tog; 
 
-/* Special case case of no radio group. */
+/* Special case of no radio group. */
 
   if ( (group = GetRadioGroup(radio_group)) == NULL) {
     local_tog = (ToggleWidget) radio_group;
     if ( (local_tog->toggle.radio_data == radio_data) )     
       if (!local_tog->command.set) {
-	ToggleSet((Widget) local_tog, NULL, NULL, 0);
-	Notify((Widget) local_tog, NULL, NULL, 0);
+	ToggleSet((Widget) local_tog, (XEvent *)NULL, (String *)NULL, (Cardinal *)0);
+	Notify((Widget) local_tog, (XEvent *)NULL, (String *)NULL, (Cardinal *)NULL);
       }
     return;
   }
@@ -572,8 +573,8 @@ XtPointer radio_data;
     local_tog = (ToggleWidget) group->widget;
     if ( (local_tog->toggle.radio_data == radio_data) ) {
       if (!local_tog->command.set) { /* if not already set. */
-	ToggleSet((Widget) local_tog, NULL, NULL, 0);
-	Notify((Widget) local_tog, NULL, NULL, 0);
+	ToggleSet((Widget) local_tog, (XEvent *)NULL, (String *)NULL, (Cardinal *)NULL);
+	Notify((Widget) local_tog, (XEvent *)NULL, (String *)NULL, (Cardinal *)NULL);
       }
       return;			/* found it, done */
     }
@@ -603,8 +604,9 @@ Widget radio_group;
   if (local_tog->command.set) {
     class = (ToggleWidgetClass) local_tog->core.widget_class;
     class->toggle_class.Reset(radio_group, NULL, NULL, 0);
-    Notify(radio_group, NULL, NULL, 0);
+    Notify(radio_group, (XEvent *)NULL, (String *)NULL, (Cardinal *)NULL);
   }
   if ( GetRadioGroup(radio_group) == NULL) return;
   TurnOffRadioSiblings(radio_group);
 }
+
