@@ -1,8 +1,27 @@
-/* $XConsortium: Viewport.c,v 1.68 91/07/24 18:56:11 converse Exp $ */
+/* $TOG: Viewport.c /main/72 1998/02/06 12:53:07 kaleb $ */
 
-/***********************************************************
-Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
-and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
+/***********************************************************************
+
+Copyright 1987, 1988, 1994, 1998  The Open Group
+
+All Rights Reserved.
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+Except as contained in this notice, the name of The Open Group shall not be
+used in advertising or otherwise to promote the sale, use or other dealings
+in this Software without prior written authorization from The Open Group.
+
+
+Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
@@ -10,7 +29,7 @@ Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, 
 provided that the above copyright notice appear in all copies and that
 both that copyright notice and this permission notice appear in 
-supporting documentation, and that the names of Digital or MIT not be
+supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
 software without specific, written prior permission.  
 
@@ -22,7 +41,9 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
-******************************************************************/
+This file contains modifications for XawPlus, Roland Krause 2000
+
+**************************************************************************/
 
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
@@ -67,7 +88,8 @@ static Boolean GetGeometry();
 static void ComputeWithForceBars();
 static void ScrollUpDownProc(), ThumbProc();
 
-static void Initialize(), ConstraintInitialize(), Realize(), Resize(), ChangeManaged();
+static void Initialize(), ConstraintInitialize(),
+	    Realize(), Resize(), ChangeManaged();
 static Boolean SetValues(), Layout();
 static XtGeometryResult GeometryManager(), PreferredGeometry();
 
@@ -156,13 +178,13 @@ static Widget CreateScrollbar(w, horizontal)
     ViewportConstraints constraints =
 	(ViewportConstraints)clip->core.constraints;
     static Arg barArgs[] = {
-	{XtNorientation,	(XtArgVal)NULL},
-	{XtNlength, 		(XtArgVal)NULL},
-	{XtNleft, 		(XtArgVal)NULL},
-	{XtNright, 		(XtArgVal)NULL},
-	{XtNtop, 		(XtArgVal)NULL},
-	{XtNbottom, 		(XtArgVal)NULL},
-	{XtNmappedWhenManaged, 	False}
+	{XtNorientation,       (XtArgVal) 0},
+	{XtNlength,            (XtArgVal) 0},
+	{XtNleft,              (XtArgVal) 0},
+	{XtNright,             (XtArgVal) 0},
+	{XtNtop,               (XtArgVal) 0},
+	{XtNbottom,            (XtArgVal) 0},
+	{XtNmappedWhenManaged, (XtArgVal) False},
     };
     Widget bar;
 
@@ -179,7 +201,7 @@ static Widget CreateScrollbar(w, horizontal)
     XtSetArg(barArgs[5], XtNbottom,
 	     (horizontal && !w->viewport.usebottom) ? XtChainTop: XtChainBottom);
 
-    bar = XtCreateWidget( (horizontal ? "horizontal" : "vertical"),
+    bar = XtCreateWidget((horizontal ? "horizontal" : "vertical"),
 			  scrollbarWidgetClass, (Widget)w,
 			  barArgs, XtNumber(barArgs) );
     XtAddCallback( bar, XtNscrollProc, ScrollUpDownProc, (XtPointer)w );
@@ -212,7 +234,7 @@ Cardinal *n_args;
 {
     ViewportWidget w = (ViewportWidget)new;
     static Arg clip_args[8];
-    Cardinal num_args;
+    Cardinal arg_cnt;
     Widget h_bar, v_bar;
     Dimension clip_height, clip_width;
 
@@ -226,27 +248,22 @@ Cardinal *n_args;
     w->viewport.child = (Widget) NULL;
     w->viewport.horiz_bar = w->viewport.vert_bar = (Widget)NULL;
 
-/*
- * Install the Action procedures
- */
-   
-
 /* 
  * Create Clip Widget.
  */
 
-    num_args = 0;
-    XtSetArg(clip_args[num_args], XtNbackgroundPixmap, None); num_args++;
-    XtSetArg(clip_args[num_args], XtNborderWidth, 0); num_args++;
-    XtSetArg(clip_args[num_args], XtNleft, XtChainLeft); num_args++;
-    XtSetArg(clip_args[num_args], XtNright, XtChainRight); num_args++;
-    XtSetArg(clip_args[num_args], XtNtop, XtChainTop); num_args++;
-    XtSetArg(clip_args[num_args], XtNbottom, XtChainBottom); num_args++;
-    XtSetArg(clip_args[num_args], XtNwidth, w->core.width); num_args++;
-    XtSetArg(clip_args[num_args], XtNheight, w->core.height); num_args++;
+    arg_cnt = 0;
+    XtSetArg(clip_args[arg_cnt], XtNbackgroundPixmap, None); arg_cnt++;
+    XtSetArg(clip_args[arg_cnt], XtNborderWidth, 0); arg_cnt++;
+    XtSetArg(clip_args[arg_cnt], XtNleft, XtChainLeft); arg_cnt++;
+    XtSetArg(clip_args[arg_cnt], XtNright, XtChainRight); arg_cnt++;
+    XtSetArg(clip_args[arg_cnt], XtNtop, XtChainTop); arg_cnt++;
+    XtSetArg(clip_args[arg_cnt], XtNbottom, XtChainBottom); arg_cnt++;
+    XtSetArg(clip_args[arg_cnt], XtNwidth, w->core.width); arg_cnt++;
+    XtSetArg(clip_args[arg_cnt], XtNheight, w->core.height); arg_cnt++;
 
     w->viewport.clip = XtCreateManagedWidget("clip", widgetClass, new,
-					     clip_args, num_args);
+					     clip_args, arg_cnt);
 
     if (!w->viewport.forcebars) 
         return;		 /* If we are not forcing the bars then we are done. */
@@ -276,15 +293,17 @@ Cardinal *n_args;
 	  (int)(v_bar->core.height + v_bar->core.border_width)) )
         clip_height -= v_bar->core.height + v_bar->core.border_width;
 
-    num_args = 0;
-    XtSetArg(clip_args[num_args], XtNwidth, clip_width); num_args++;
-    XtSetArg(clip_args[num_args], XtNheight, clip_height); num_args++;
-    XtSetValues(w->viewport.clip, clip_args, num_args);
+    arg_cnt = 0;
+    XtSetArg(clip_args[arg_cnt], XtNwidth, clip_width); arg_cnt++;
+    XtSetArg(clip_args[arg_cnt], XtNheight, clip_height); arg_cnt++;
+    XtSetValues(w->viewport.clip, clip_args, arg_cnt);
 }
 
 /* ARGSUSED */
-static void ConstraintInitialize(request, new)
+static void ConstraintInitialize(request, new, args, num_args)
     Widget request, new;
+    ArgList args;
+    Cardinal *num_args;
 {
     ((ViewportConstraints)new->core.constraints)->viewport.reparented = False;
 }
@@ -295,8 +314,8 @@ static void Realize(widget, value_mask, attributes)
     XSetWindowAttributes *attributes;
 {
     ViewportWidget w = (ViewportWidget)widget;
-    register Widget child = w->viewport.child;
-    register Widget clip = w->viewport.clip;
+    Widget child = w->viewport.child;
+    Widget clip = w->viewport.clip;
 
     *value_mask |= CWBitGravity;
     attributes->bit_gravity = NorthWestGravity;
@@ -314,7 +333,7 @@ static void Realize(widget, value_mask, attributes)
     }
 }
 
-
+/* ARGSUSED */
 static Boolean SetValues(current, request, new, args, num_args)
 Widget current, request, new;
 ArgList args;
@@ -340,9 +359,9 @@ static void ChangeManaged(widget)
     Widget widget;
 {
     ViewportWidget w = (ViewportWidget)widget;
-    register int num_children = w->composite.num_children;
-    register Widget child, *childP;
-    register int i;
+    int num_children = w->composite.num_children;
+    Widget child, *childP;
+    int i;
 
     child = (Widget)NULL;
     for (childP=w->composite.children, i=0; i < num_children; childP++, i++) {
@@ -416,8 +435,8 @@ static void SetBar(w, top, length, total)
 static void RedrawThumbs(w)
   ViewportWidget w;
 {
-    register Widget child = w->viewport.child;
-    register Widget clip = w->viewport.clip;
+    Widget child = w->viewport.child;
+    Widget clip = w->viewport.clip;
 
     if (w->viewport.horiz_bar != (Widget)NULL)
 	SetBar( w->viewport.horiz_bar, -(child->core.x),
@@ -437,8 +456,8 @@ static void SendReport (w, changed)
     XawPannerReport rep;
 
     if (w->viewport.report_callbacks) {
-	register Widget child = w->viewport.child;
-	register Widget clip = w->viewport.clip;
+	Widget child = w->viewport.child;
+	Widget clip = w->viewport.clip;
 
 	rep.changed = changed;
 	rep.slider_x = -child->core.x;	/* child is canvas */
@@ -457,8 +476,8 @@ static void MoveChild(w, x, y)
     ViewportWidget w;
     Position x, y;
 {
-    register Widget child = w->viewport.child;
-    register Widget clip = w->viewport.clip;
+    Widget child = w->viewport.child;
+    Widget clip = w->viewport.clip;
 
     /* make sure we never move past right/bottom borders */
     if (-x + (int)clip->core.width > (int)child->core.width)
@@ -484,8 +503,8 @@ static void ComputeLayout(widget, query, destroy_scrollbars)
     Boolean destroy_scrollbars;	/* destroy un-needed scrollbars? */
 {
     ViewportWidget w = (ViewportWidget)widget;
-    register Widget child = w->viewport.child;
-    register Widget clip = w->viewport.clip;
+    Widget child = w->viewport.child;
+    Widget clip = w->viewport.clip;
     ViewportConstraints constraints
 	= (ViewportConstraints)clip->core.constraints;
     Boolean needshoriz, needsvert;
@@ -604,17 +623,17 @@ static void ComputeLayout(widget, query, destroy_scrollbars)
 	XRaiseWindow( XtDisplay(clip), XtWindow(clip) );
 
     XtMoveWidget( clip,
-		  needsvert ? (w->viewport.useright ? 0 :
+		  (Position) (needsvert ? (w->viewport.useright ? 0 :
 			       w->viewport.vert_bar->core.width +
-			       w->viewport.vert_bar->core.border_width) : 0,
-		  needshoriz ? (w->viewport.usebottom ? 0 :
+			       w->viewport.vert_bar->core.border_width) : 0),
+		  (Position) (needshoriz ? (w->viewport.usebottom ? 0 :
 				w->viewport.horiz_bar->core.height +
-			        w->viewport.horiz_bar->core.border_width) : 0);
+			        w->viewport.horiz_bar->core.border_width) : 0));
     XtResizeWidget( clip, (Dimension)clip_width,
 		    (Dimension)clip_height, (Dimension)0 );
 	
     if (w->viewport.horiz_bar != (Widget)NULL) {
-	register Widget bar = w->viewport.horiz_bar;
+	Widget bar = w->viewport.horiz_bar;
 	if (!needshoriz) {
 	    constraints->form.vert_base = (Widget)NULL;
 	    if (destroy_scrollbars) {
@@ -623,13 +642,13 @@ static void ComputeLayout(widget, query, destroy_scrollbars)
 	    }
 	}
 	else {
-	    register int bw = bar->core.border_width;
-	    XtResizeWidget( bar, clip_width, bar->core.height, bw );
+	    int bw = bar->core.border_width;
+	    XtResizeWidget( bar, (Dimension) clip_width, bar->core.height, (Dimension) bw );
 	    XtMoveWidget( bar,
-			  ((needsvert && !w->viewport.useright)
+			  (Position)((needsvert && !w->viewport.useright)
 			   ? w->viewport.vert_bar->core.width
 			   : -bw),
-			  (w->viewport.usebottom
+			  (Position)(w->viewport.usebottom
 			    ? w->core.height - bar->core.height - bw
 			    : -bw) );
 	    XtSetMappedWhenManaged( bar, True );
@@ -637,7 +656,7 @@ static void ComputeLayout(widget, query, destroy_scrollbars)
     }
 
     if (w->viewport.vert_bar != (Widget)NULL) {
-	register Widget bar = w->viewport.vert_bar;
+	Widget bar = w->viewport.vert_bar;
 	if (!needsvert) {
 	    constraints->form.horiz_base = (Widget)NULL;
 	    if (destroy_scrollbars) {
@@ -646,13 +665,13 @@ static void ComputeLayout(widget, query, destroy_scrollbars)
 	    }
 	}
 	else {
-	    register int bw = bar->core.border_width;
-	    XtResizeWidget( bar, bar->core.width, clip_height, bw );
+	    int bw = bar->core.border_width;
+	    XtResizeWidget( bar, bar->core.width, (Dimension)clip_height, (Dimension)bw );
 	    XtMoveWidget( bar,
-			  (w->viewport.useright
+			  (Position)(w->viewport.useright
 			   ? w->core.width - bar->core.width - bw 
 			   : -bw),
-			  ((needshoriz && !w->viewport.usebottom)
+			  (Position)((needshoriz && !w->viewport.usebottom)
 			    ? w->viewport.horiz_bar->core.height
 			    : -bw) );
 	    XtSetMappedWhenManaged( bar, True );
@@ -689,7 +708,7 @@ XtWidgetGeometry * intended;
 int *clip_width, *clip_height;
 {
     ViewportWidget w = (ViewportWidget)widget;
-    register Widget child = w->viewport.child;
+    Widget child = w->viewport.child;
     XtWidgetGeometry preferred;
 
 /*
@@ -780,7 +799,7 @@ static void ScrollUpDownProc(widget, closure, call_data)
     XtPointer call_data;
 {
     ViewportWidget w = (ViewportWidget)closure;
-    register Widget child = w->viewport.child;
+    Widget child = w->viewport.child;
     int pix = (int)call_data;
     Position x, y;
 
@@ -793,13 +812,14 @@ static void ScrollUpDownProc(widget, closure, call_data)
 
 
 /* ARGSUSED */
-static void ThumbProc(widget, closure, percent)
+static void ThumbProc(widget, closure, call_data)
     Widget widget;
     XtPointer closure;
-    float *percent;
+    XtPointer call_data;
 {
     ViewportWidget w = (ViewportWidget)closure;
-    register Widget child = w->viewport.child;
+    Widget child = w->viewport.child;
+    float *percent = (float *) call_data;
     Position x, y;
 
     if (child == NULL) return;	/* no child to scroll. */
@@ -1009,7 +1029,7 @@ static Boolean GetGeometry(w, width, height)
 
     result = XtMakeGeometryRequest(w, &geometry, &return_geom);
     if (result == XtGeometryAlmost)
-	result = XtMakeGeometryRequest(w, &return_geom, NULL);
+	result = XtMakeGeometryRequest(w, &return_geom, (XtWidgetGeometry *)NULL);
 
     return (result == XtGeometryYes);
 }
@@ -1046,7 +1066,7 @@ XawViewportSetLocation (gw, xoff, yoff)
 #endif
 {
     ViewportWidget w = (ViewportWidget) gw;
-    register Widget child = w->viewport.child;
+    Widget child = w->viewport.child;
     Position x, y;
 
     if (xoff > 1.0)			/* scroll to right */
